@@ -33,7 +33,7 @@
  *
  */
 
-/* $Id: codetable.c,v 1.1.1.1.2.1 2002/04/12 22:22:18 famao Exp $ */
+/* $Id: codetable.c,v 1.7 2002/11/09 16:56:47 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -48,6 +48,7 @@
 #include "skk_xaux.h"
 #include "codetable.h"
 #include "support.h"
+#include "config.h"
 
 #ifndef SKK_AUX_CODE_DEBUG
 #include "xaux_common.h"
@@ -237,7 +238,11 @@ table_clicked (GtkWidget *widget, gpointer user_data)
 		g_message ("%s\n",label);
 #else
 	if (label && *label) {
+#ifdef USE_GTK2
+		send_str[0] = g_locale_from_utf8 (label, -1, NULL, NULL, NULL);
+#else
 		send_str[0] = g_strdup (label);
+#endif
 		send_int[0] = SKK_AUX_CODE_DEFINED;
 		skk_aux_palette_send_engine (1, send_int, 1, send_str);
 		g_free (send_str[0]);
@@ -254,6 +259,9 @@ for_each (GtkWidget *widget, gpointer user_data)
 	wchar_t wc;
 	int result;
 	CodeTable *table = (CodeTable *) user_data;
+#ifdef USE_GTK2
+	gchar *ustr;
+#endif
 	child = GTK_BIN (widget)->child;
 	if (!child)
 		return;
@@ -271,9 +279,15 @@ for_each (GtkWidget *widget, gpointer user_data)
 	result = mbtowc (&wc, text, MB_CUR_MAX);
 	if (result > 0) {
 #ifdef SKK_AUX_CODE_DEBUG
-		g_message ("%s\n",text);
+		printf ("%s\n",text);
 #endif
+#ifdef USE_GTK2
+		ustr = g_locale_to_utf8 (text, -1, NULL, NULL, NULL);
+		gtk_label_set_text (GTK_LABEL (child), ustr);
+		g_free (ustr);
+#else
 		gtk_label_set_text (GTK_LABEL (child), text);
+#endif
 	} else {
 		gtk_label_set_text (GTK_LABEL (child), NULL);
 	}
@@ -532,12 +546,19 @@ main (int argc, char *argv[])
 {
 	GtkWidget *toplevel;
 	/* force to EUC-JP */
+#ifdef HAVE_SETENV
 	setenv ("LC_ALL","ja_JP.EUC-JP",TRUE);
+#elif HAVE_PUTENV
+	putenv ("LC_ALL=ja_JP.EUC-JP");
+#endif
 	gtk_set_locale ();
 
 #ifdef ENABLE_NLS
 	bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
-	textdomain (PACKAGE);
+#ifdef USE_GTK2
+	bind_textdomain_codeset (PACKAGE, "UTF-8");
+#endif
+	textdomain (PACKAGE); 
 #endif
 	gtk_init (&argc, &argv);
 	toplevel = skk_aux_code_new ();
