@@ -33,7 +33,7 @@
  *
  */
 
-/* $Id: palette.c,v 1.3.2.1 2002/04/28 00:22:23 famao Exp $ */
+/* $Id: palette.c,v 1.9 2002/11/09 16:56:47 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -54,12 +54,18 @@
 #include "xaux_ext_common.h"
 #include "skk_xaux.h"
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "pixmaps/hira.xpm"
 #include "pixmaps/kata.xpm"
 #include "pixmaps/jisx0208.xpm"
 #include "pixmaps/jisx0201.xpm"
 #include "pixmaps/ascii.xpm"
+#ifndef notdef
 #include "pixmaps/code.xpm"
+#endif
 
 #include "codetable.h"
 #include "support.h"
@@ -74,6 +80,7 @@ xaux_class_t xaux_class = {
 
 static GtkWidget *toplevel = NULL;
 static GtkWidget *status_pixmaps[6] = { NULL, NULL, NULL, NULL, NULL, NULL};
+static gboolean is_slit = FALSE;
 static xaux_class_t *xc_ = 0;
 static int im_ = 0;
 static int ic_ = 0;
@@ -100,6 +107,13 @@ static GtkWidget *palette_new (void);
 static GtkWidget *status_menu_new (void);
 static GtkWidget *pixmap_new (GtkWidget *parent, gint type);
 
+/* WM Spec */
+static void set_palette_window_docked (GdkWindow *window);
+static void set_palette_window_sticky (void);
+static void set_palette_window_sticky_gnome (GdkWindow *window);
+
+static gboolean is_blackbox_wm (void);
+
 /* Error */
 static int palette_x_error (Display *d, XErrorEvent *error);
 
@@ -127,6 +141,7 @@ menu_detach_func (GtkWidget *attach_widget, GtkMenu *menu)
 	return;
 }
 
+#ifndef notdef
 static GtkWidget*
 pixmap_new_with_xpm (GtkWidget *parent, gchar **xpm)
 {
@@ -145,6 +160,7 @@ pixmap_new_with_xpm (GtkWidget *parent, gchar **xpm)
 	gdk_bitmap_unref (mask);
 	return pixmap;
 }
+#endif
 
 static GtkWidget* 
 pixmap_new (GtkWidget *parent, gint type)
@@ -178,7 +194,11 @@ pixmap_new (GtkWidget *parent, gint type)
 			NULL, xpm);
 	if (gdkpixmap == NULL)
 		return NULL;
+#ifndef USE_GTK2
 	pixmap = gtk_pixmap_new (gdkpixmap, mask);
+#else
+	pixmap = gtk_image_new_from_pixmap (gdkpixmap, mask);
+#endif
 	gdk_pixmap_unref (gdkpixmap);
 	gdk_bitmap_unref (mask);
 	return pixmap;
@@ -209,53 +229,111 @@ create_palette_button (gchar *text, GtkPixmap *icon)
 	return button;
 }
 
+static void
+set_palette_window_docked (GdkWindow *window)
+{
+	Atom atoms[2] = { None, None };
+	atoms[0] = XInternAtom (GDK_WINDOW_XDISPLAY (window), "_NET_WM_WINDOW_TYPE_DOCK", False);
+	XChangeProperty (GDK_WINDOW_XDISPLAY (window),
+			GDK_WINDOW_XWINDOW (window),
+			XInternAtom (GDK_WINDOW_XDISPLAY (window), "_NET_WM_WINDOW_TYPE", False),
+			XA_ATOM, 32, PropModeReplace, (guchar *)atoms, 1);
+	return;
+}
+
+static gboolean
+is_blackbox_wm (void)
+{
+	long *value;
+	Atom type_r;
+	int format_r;
+	unsigned long nitems_r, bytes_after;
+	/* 
+	 * BlackBox-based WindowManager sets _BLACKBOX_PID Property
+	 * to RootWindow
+	 */
+	Atom isbb = XInternAtom (GDK_DISPLAY (), "_BLACKBOX_PID", False);
+	if (XGetWindowProperty (GDK_DISPLAY (), GDK_ROOT_WINDOW (), 
+			isbb, 0, 128, False, XA_CARDINAL, &type_r, &format_r,
+			&nitems_r, &bytes_after,
+			(unsigned char **)&value) == Success) {
+		XFree (value);
+		if (nitems_r > 0 && type_r != None) {
+			return TRUE;
+		} else {
+			return FALSE;
+		}
+	}
+	return FALSE;
+}
+
 static GtkWidget *
 palette_new (void)
 {
 	GtkWidget *palette;
 	GtkWidget *handlebox;
+	GtkWidget *vbox;
 	GtkWidget *toolbar;
 	GtkWidget *icon;
 	GtkWidget *button;
 
 	palette = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	gtk_widget_realize (palette);
+	vbox = gtk_vbox_new (FALSE, FALSE);
 	gtk_window_set_policy (GTK_WINDOW (palette), FALSE, FALSE, TRUE);
 	handlebox = gtk_handle_box_new ();
 	gtk_widget_show (handlebox);
-	gtk_container_add (GTK_CONTAINER (palette), handlebox);
+	gtk_container_add (GTK_CONTAINER (palette), vbox);
+	gtk_widget_show (vbox);
+	gtk_box_pack_start (GTK_BOX (vbox), handlebox, FALSE, FALSE, 0);
 #ifndef USE_GTK2 
 	toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL,
 			GTK_TOOLBAR_ICONS);
 #else
 	toolbar = gtk_toolbar_new ();
+	gtk_toolbar_set_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_BOTH_HORIZ); 
 #endif
 	gtk_widget_show (toolbar);
 	gtk_container_add (GTK_CONTAINER (handlebox), toolbar);
 	icon = pixmap_new (palette, 0);
 
+#ifndef USE_GTK2
 	button = create_palette_button ("SKK", GTK_PIXMAP (icon));
+	gtk_container_set_resize_mode (GTK_CONTAINER (button), GTK_RESIZE_PARENT);
 	gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
 			GTK_TOOLBAR_CHILD_WIDGET, button, NULL, NULL, NULL, NULL, NULL, NULL);
-#if 0
-	gtk_button_set_relief (GTK_BUTTON (button), GTK_TOOLBAR (toolbar)->relief);
+	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
+#else
+	button = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
+			GTK_TOOLBAR_CHILD_BUTTON, NULL, "SKK", NULL, NULL, icon,
+			NULL, NULL);
 #endif
 	gtk_signal_connect (GTK_OBJECT (button), "clicked",
 			GTK_SIGNAL_FUNC (status_button_clicked), NULL);
 	gtk_object_set_data (GTK_OBJECT (palette), "status_button",
 			(gpointer) button);
-	gtk_widget_set_parent (button, toolbar);
-	gtk_widget_queue_resize (button);
-	gtk_widget_queue_resize (toolbar);
 	gtk_widget_show (button);
 
 	gtk_object_set_data (GTK_OBJECT (palette), "toolbar", (gpointer) toolbar);
 
+#ifndef notdef
 	icon = pixmap_new_with_xpm (palette, code_xpm);
+	button = create_palette_button (NULL, GTK_PIXMAP (icon));
+#else
+	icon = gtk_image_new_from_stock (GTK_STOCK_INDEX, GTK_ICON_SIZE_BUTTON);
+#endif
+#if 0
 	button = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
 			GTK_TOOLBAR_CHILD_BUTTON, NULL, NULL, "Code Table", NULL, icon, 
 			GTK_SIGNAL_FUNC (code_button_clicked), NULL);
-
+#endif
+	button = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
+			GTK_TOOLBAR_CHILD_WIDGET, button, NULL, "Code Table", NULL, NULL,
+			NULL, NULL);
+	gtk_signal_connect (GTK_OBJECT (button), "clicked",
+			GTK_SIGNAL_FUNC (code_button_clicked), NULL);
+	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
+	gtk_widget_show (button);		
 	return palette;
 }
 
@@ -264,6 +342,8 @@ palette_reposition (GtkWidget *widget)
 {
 	GtkRequisition sizereq;
 	gint width,height;
+	if (is_slit)
+		return;
 	if (!GTK_WIDGET_REALIZED (widget))
 		return;
 	if (!GTK_IS_WINDOW (widget))
@@ -374,10 +454,13 @@ static void
 status_changed (gchar *text, gint type)
 {
 	GtkWidget *toolbar;
-	GtkWidget *item;
 	GtkToolbarChild *child;
 	GdkBitmap *mask;
 	GdkPixmap *pix;
+	GtkWidget *item;
+#ifdef USE_GTK2
+	gchar *ustr;
+#endif
 	if (!text)
 		return;
 
@@ -398,17 +481,36 @@ status_changed (gchar *text, gint type)
 		return;
 	if (child->widget) {
 		/* label */
+#ifdef USE_GTK2
+		item = child->label;
+#else
 		item = (GtkWidget *)gtk_object_get_data (GTK_OBJECT (child->widget), "label");
+#endif
 		if (item) {
+#ifdef USE_GTK2
+			ustr = g_locale_to_utf8 (text, -1, NULL, NULL, NULL);
+			gtk_label_set_text (GTK_LABEL (item), ustr);
+			g_free (ustr);
+#else
 			gtk_label_set_text (GTK_LABEL (item), text);
+#endif
 		}
+#ifdef USE_GTK2
+		item = child->icon;
+#else
 		item = (GtkWidget *)gtk_object_get_data (GTK_OBJECT (child->widget), "icon");
+#endif
 		if (item) {
 			if (status_pixmaps[type] == NULL) {
 				status_pixmaps[type] = pixmap_new (toplevel, type);
 			}
+#ifndef USE_GTK2
 			gtk_pixmap_get (GTK_PIXMAP (status_pixmaps[type]), &pix, &mask);
 			gtk_pixmap_set (GTK_PIXMAP (item), pix, mask);
+#else
+			gtk_image_get_pixmap (GTK_IMAGE (status_pixmaps[type]), &pix, &mask);
+			gtk_image_set_from_pixmap (GTK_IMAGE (item), pix, mask);
+#endif
 		}
 	}
 	/* RePositioning */
@@ -423,7 +525,18 @@ show_codetable (void)
 	code = (GtkWidget *) gtk_object_get_data (GTK_OBJECT (toplevel), "code");
 	if (!code)
 		return;
+	/* 
+	  in the case of having codetable in DISPLAY,
+	  raise window
+	 */ 
+	if (GTK_WIDGET_VISIBLE (code)) {
+		gdk_window_raise (code->window);
+		return;
+	}
 	gtk_widget_show (code);
+	/* TODO need it ? */
+	gdk_window_raise (code->window);
+	return;
 }
 
 static void
@@ -641,25 +754,99 @@ skk_aux_palette_send_engine (gint int_count, gint *integers, gint str_count, gch
 	return;
 }
 
+static void
+set_palette_window_sticky (void)
+{
+	/*
+	 * Sticky Support
+	 * See http://www.freedesktop.org/standards/wm-spec/x186.html for more detail.
+	 */
+	Atom wm_state = XInternAtom (GDK_DISPLAY (), "_NET_WM_STATE", FALSE);
+	Atom actual_state_sticky = XInternAtom (GDK_DISPLAY (), "_NET_WM_STATE_STICKY", FALSE);
+	Atom actual_state_skip_taskbar = XInternAtom (GDK_DISPLAY (), "_NET_WM_STATE_SKIP_TASKBAR", FALSE);
+	Atom actual_state_skip_pager = XInternAtom (GDK_DISPLAY (), "_NET_WM_STATE_SKIP_PAGER", FALSE);
+	XClientMessageEvent xev;
+
+	/* sending wm_state */
+	xev.type = ClientMessage;
+	xev.window = GDK_WINDOW_XWINDOW (toplevel->window);
+	xev.message_type = (Atom)wm_state;
+	xev.format = 32;
+	xev.data.l[0] = 1; /* _NET_WM_STATE_ADD */
+	xev.data.l[1] = actual_state_sticky;
+	xev.data.l[2] = actual_state_skip_taskbar;
+	xev.data.l[3] = actual_state_skip_pager;
+	xev.data.l[4] = 0;
+	XSendEvent (GDK_DISPLAY (), GDK_ROOT_WINDOW (), False, SubstructureNotifyMask, (XEvent *) &xev);
+
+	return;
+	
+}
+
+static void
+set_palette_window_sticky_gnome (GdkWindow *window)
+{
+	/* WIN_STATE_STICKY */
+	long state_flags = (1<<0);
+	/* WIN_HINTS_SKIP_FOCUS | WIN_HINTS_SKIP_TASKBAR */
+	long hints_flags = (1<<1) | (1<<2);
+	Atom gnome_sticky = XInternAtom (GDK_DISPLAY (), "_WIN_STATE", FALSE);
+	Atom gnome_hints = XInternAtom (GDK_DISPLAY (), "_WIN_HINTS", FALSE);
+	if (!window)
+		return;
+	/* Gnome Support */
+	XChangeProperty (GDK_DISPLAY (), GDK_WINDOW_XWINDOW (window),
+			gnome_sticky, 
+			XA_CARDINAL,
+			32, 
+			PropModeReplace, (unsigned char *)&state_flags, 1);
+
+	XChangeProperty (GDK_DISPLAY (), GDK_WINDOW_XWINDOW (window),
+			gnome_hints,
+			XA_CARDINAL,
+			32,
+			PropModeReplace, (unsigned char *)&hints_flags, 1);
+
+}
+
 int
 main (int argc, char *argv[])
 {
 	GtkWidget *menu;
 	GtkWidget *code;
 	/* force to EUC-JP */
+#ifdef HAVE_SETENV
 	setenv ("LC_ALL","ja_JP.eucJP",True);
-
+#elif HAVE_PUTENV
+	putenv ("LC_ALL=ja_JP.eucJP");
+#endif
 	gtk_set_locale ();
 
 #ifdef ENABLE_NLS
 	bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
+#ifdef USE_GTK2
+	bind_textdomain_codeset (PACKAGE, "UTF-8");
+#endif
 	textdomain (PACKAGE);
 #endif
 
 	gtk_init (&argc, &argv);
 	toplevel = palette_new ();
 	gdk_window_set_decorations (toplevel->window,0);
-	gdk_window_set_override_redirect (toplevel->window, FALSE);
+	/* Dock Support */
+	if (getenv ("IIIMF_SKK_WITH_SLIT") || is_blackbox_wm ()) {
+		is_slit = TRUE;
+	}	
+	if (is_slit) {
+		XWMHints hints;
+		hints.flags = StateHint;
+		hints.initial_state = WithdrawnState;
+		XSetWMHints (GDK_DISPLAY (), GDK_WINDOW_XWINDOW (toplevel->window), &hints);
+	} else {
+		set_palette_window_docked (toplevel->window);
+		set_palette_window_sticky ();
+		set_palette_window_sticky_gnome (toplevel->window);
+	}	
 	gtk_widget_show (toplevel);
 	palette_reposition (toplevel);
 	menu = status_menu_new ();
@@ -667,7 +854,7 @@ main (int argc, char *argv[])
 	gtk_object_set_data (GTK_OBJECT (toplevel), "menu", (gpointer)menu);
 	code = skk_aux_code_new ();
 	gtk_object_set_data (GTK_OBJECT (toplevel), "code", (gpointer)code);
-
+	gdk_window_set_transient_for (toplevel->window, code->window);
 	gdk_window_add_filter (toplevel->window, ext_event_handler, 0);
 	gtk_widget_hide (toplevel);
 	if (xaux_ext_init_classes (GDK_DISPLAY (),
