@@ -33,7 +33,7 @@
  *
  */
 
-/* $Id: xaux_ext_common.c,v 1.1.1.1 2002/04/06 02:23:31 famao Exp $ */
+/* $Id: xaux_ext_common.c,v 1.4 2002/11/09 16:56:47 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -83,7 +83,7 @@ Sun Microsystems, Inc. or its licensors is granted.
 */
 
 /*
- * $Id: xaux_ext_common.c,v 1.1.1.1 2002/04/06 02:23:31 famao Exp $
+ * $Id: xaux_ext_common.c,v 1.4 2002/11/09 16:56:47 famao Exp $
  * Modified by Motonobu Ichimura <famao@kondara.org>
  */
 
@@ -107,7 +107,9 @@ Sun Microsystems, Inc. or its licensors is granted.
 #include "xaux_common.h"
 #include "xaux_ext_common.h"
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
 
 static Bool xaux_xs_send_message (Display *display, xaux_class_t *xc, int im_id, int ic_id,
 		aux_ext_data_type_t type, Atom atom);
@@ -885,10 +887,10 @@ HelpInvoke(
       return;
     }
 
-#ifdef linux
-    pid = fork();
+#ifdef HAVE_FORK1
+	pid = fork1();
 #else
-    pid = fork1();
+	pid = fork ();
 #endif
 
     if (getenv("HTT_HELP_BROWSER")){
@@ -1087,7 +1089,12 @@ IIimpConvertToUTF16(char *from_buf, size_t from_left,
   }
 
   if (i_conv1 == NULL && !skip_native_to_utf8_conv) {
+#ifdef HAVE_LANGINFO_CODESET
     char *encoding = nl_langinfo(CODESET);
+#else
+	/* Dirty Hack */
+	char *encoding = "EUC-JP";
+#endif
     if (!strcmp(encoding, "UTF-8")) {
       skip_native_to_utf8_conv = True;
     } else {
@@ -1214,7 +1221,11 @@ IIimpConvertFromUTF16(char *from_buf, size_t from_left,
   }
 
   if (i_conv2_FromUTF16 == NULL && !skip_native_to_utf8_conv) {
+#ifdef HAVE_LANGINFO_CODESET
     char *encoding = nl_langinfo(CODESET);
+#else
+	char *encoding = "EUC-JP";
+#endif
     if (!strcmp(encoding, "UTF-8")) {
       skip_native_to_utf8_conv = True;
     } else {
