@@ -32,7 +32,7 @@
  *
  */
 
-/* $Id: skkbuffer.c,v 1.14.2.4 2002/04/24 00:27:55 famao Exp $ */
+/* $Id: skkbuffer.c,v 1.21 2002/11/09 16:56:48 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -53,6 +53,7 @@
 /* buffer operation */
 static gchar* get_buf (SkkBuffer *buf, gint ch, gboolean lower);
 static gchar* get_translate_buf (SkkBuffer *buf);
+static gchar* get_direction_word (SkkBuffer *buf);
 static gchar* create_candidate (SkkBuffer *buf);
 
 static gboolean delete_backward_okurigana (SkkBuffer *buf);
@@ -83,6 +84,7 @@ static void set_result_buf (SkkBuffer *buf, const gchar *value);
 static void set_preedit_buf (SkkBuffer *buf, const gchar *value);
 static void set_buf (SkkBuffer *buf, const gchar *value);
 static void set_direction_word (SkkBuffer *buf, const gchar *value);
+static void set_numb (SkkBuffer *buf, const gchar *value);
 static void set_lasth (SkkBuffer *buf);
 
 static void add_okurigana (SkkBuffer *buf, const gchar *append);
@@ -91,6 +93,7 @@ static void add_direction_word (SkkBuffer *buf, const gchar *append);
 static void add_direction_word_int (SkkBuffer *buf, const gchar *append);
 static void add_result_buf (SkkBuffer *buf, const gchar *append);
 
+
 /* change category */
 static void change_category (SkkBuffer *buf);
 
@@ -99,25 +102,148 @@ static gboolean can_clear (SkkBuffer *buf);
 static gboolean can_commit (SkkBuffer *buf);
 static gboolean has_preedit (SkkBuffer *buf);
 
-typedef struct _SkkBufFunc {
-	SkkBufListener listener;
-	gpointer user_data;
-}SkkBufFunc;
+/* cursor */
+static void move_left (SkkBuffer *buf);
+static void move_right (SkkBuffer *buf);
+static gboolean separate_string_by_cursor (SkkBuffer *buf, const gchar *string, gchar **beforep, gchar **afterp);
 
-typedef struct _SkkAddDictFunc {
+/* emit signal */
+static void preedit_emit (SkkBuffer *buf);
+static void adddict_emit (SkkBuffer *buf);
+static void result_emit (SkkBuffer *buf);
+static void codetable_emit (SkkBuffer *buf);
+static void lookup_emit (SkkBuffer *buf);
+
+
+typedef struct _SkkAddDictFunc SkkAddDictFunc;
+typedef struct _SkkBufFunc SkkBufFunc;
+typedef struct _SkkCodeTableFunc SkkCodeTableFunc;
+typedef struct _SkkLookupFunc SkkLookupFunc;
+
+
+struct _SkkAddDictFunc {
 	SkkAddDictListener listener;
 	gpointer user_data;
-}SkkAddDictFunc;
+};
 
-typedef struct _SkkCodeTableFunc {
+struct _SkkBufFunc {
+	SkkBufListener listener;
+	gpointer user_data;
+};
+
+struct _SkkCodeTableFunc {
 	SkkCodeTableListener listener;
 	gpointer user_data;
-}SkkCodeTableFunc;
+};
 
-typedef struct _SkkLookupFunc {
+struct _SkkLookupFunc {
 	SkkLookupListener listener;
 	gpointer user_data;
-}SkkLookupFunc;
+};
+
+static gchar *
+get_direction_word (SkkBuffer *buf)
+{
+	gchar *query = NULL;
+	gchar *jisx0201 = NULL;
+	gchar *direction = NULL;
+	if (!buf)
+		return NULL;
+
+	separate_string_by_cursor (buf, skkbuf_d (buf), &direction, NULL);
+
+	if (skk_buffer_get_katakana (buf) || skk_buffer_get_jisx0201_katakana (buf)) {
+		if (skk_mode_get_jisx0201_katakana (skkbuf_mode (buf))) {
+			jisx0201 = skk_conv_jisx0201_kana_to_katakana (direction ?
+					direction : skkbuf_d (buf));
+		}
+		query = skk_conv_katakana_to_hiragana (jisx0201 ? jisx0201:
+				(direction ? direction : skkbuf_d (buf)));
+		if (jisx0201)
+			g_free (jisx0201);
+		if (direction)
+			g_free (direction);
+	} else {
+		query = direction;
+	}
+	return query;
+}
+
+static void
+move_right (SkkBuffer *buf)
+{
+	SkkCursor *cursor = skkbuf_cursor (buf);
+	if (!skkbuf_p (buf)) {
+		return;
+	}
+	if (skk_buffer_get_query_status (buf) != QUERY_NORMAL) {
+		return;
+	}
+	if (cursor->caret > 0) {
+		cursor->caret--;
+		set_buf (buf, NULL);
+		preedit_emit (buf);
+	}
+}
+
+static void
+move_left (SkkBuffer *buf)
+{
+	SkkCursor *cursor = skkbuf_cursor (buf);
+	gint len;
+	if (!skkbuf_p (buf)) {
+		return;
+	}
+	if (skk_buffer_get_query_status (buf) != QUERY_NORMAL) {
+		return;
+	}
+	len = skk_utils_strlen (skkbuf_p (buf));
+	if (cursor->caret < len) {
+		cursor->caret++;
+		set_buf (buf, NULL);
+		preedit_emit (buf);
+	}
+	return;
+}
+
+static gboolean
+separate_string_by_cursor (SkkBuffer *buf, const gchar *string, gchar **beforep, gchar **afterp)
+{
+	SkkCursor *cursor;
+	gint offset;
+	gint len;
+	const gchar *after;
+	if (!buf || !string)
+		return FALSE;
+	cursor = skkbuf_cursor (buf);
+	after = string;
+	if (!cursor || !cursor->caret) {
+		return FALSE;
+	}
+	len = skk_utils_last_charbytes_nth_all (string, cursor->caret);
+	offset = strlen (string) - len;
+	if (offset == 0) {
+		if (beforep) {
+			*beforep = NULL;
+		}
+		if (afterp) {
+			*afterp = g_strdup (string);
+		}
+		return TRUE;
+	} else {
+		after += offset;
+		if (beforep) {
+			*beforep = g_strndup (string, offset);
+			g_message ("beforep %s", *beforep);
+		}
+		if (afterp) {
+			*afterp = g_strdup (after);
+			g_message ("after %s", after);
+		}
+		return TRUE;
+	}
+	return FALSE;
+}
 
 static void
 lookup_emit (SkkBuffer *buf)
@@ -212,6 +338,16 @@ preedit_emit (SkkBuffer *buf)
 		g_free (preedit);
 		preedit = NULL;
 	}
+	if (skk_buffer_get_query_status (buf) == QUERY_DONE) {
+		skkbuf_cursor (buf)->revPos = 2;
+		if (skkbuf_cursor (buf)->caret == 0) {
+			skkbuf_cursor (buf)->revLen = skkbuf_plen (buf);
+		}
+		g_message ("revLen %d", skkbuf_cursor (buf)->revLen);
+	} else {
+		skkbuf_cursor (buf)->revPos = 0;
+		skkbuf_cursor (buf)->revLen = 0;
+	}
 	for (tmp_list = buf->preedit_listener; tmp_list; tmp_list = g_list_next (tmp_list)) {
 		func = (SkkBufFunc *)tmp_list->data;
 		if (func) {
@@ -290,9 +426,9 @@ change_category (SkkBuffer *buf)
 			 * otherwise katakana to hiragana
 			 */
 			if (skk_buffer_get_katakana (buf)) {
-				result = skkconv_katakana_to_hiragana (skkbuf_p (buf));
+				result = skk_conv_katakana_to_hiragana (skkbuf_p (buf));
 			} else {
-				result = skkconv_hiragana_to_katakana (skkbuf_p (buf));
+				result = skk_conv_hiragana_to_katakana (skkbuf_p (buf));
 			}
 			/* now we have a translated value, so can clear */
 			skk_buffer_clear (buf);
@@ -314,7 +450,7 @@ change_category (SkkBuffer *buf)
 				return;
 			}
 			/* translate preedit_buf to JISX0208_LATIN */
-			result = skkconv_get_jisx0208_latin_from_str (skkbuf_p (buf), TRUE);
+			result = skk_conv_get_jisx0208_latin_from_str (skkbuf_p (buf), TRUE);
 			/* now we have a translated value, so can clear */
 			skk_buffer_clear (buf);
 			/* set result */
@@ -346,6 +482,8 @@ query_num (SkkBuffer *buf)
 		return;
 	ret = skk_num_translate (candidate, buf->number_list, buf->has_num, &type);
 	if (ret) {
+		/* store candidate */
+		set_numb (buf, candidate);	
 		if (type == 4) {
 			/* in the case of #4, retry */
 			/* reset query item */
@@ -374,7 +512,6 @@ query_num (SkkBuffer *buf)
 					/* FIXME */
 					buf->candidate_list =
 						g_list_insert (buf->candidate_list, tmp->data , count);
-					g_message ("candidate %s", ((SkkDictItem *)tmp->data)->candidate);
 					count++;
 				}
 				buf->candidate_max += list_num;
@@ -399,18 +536,42 @@ static void
 query_dict (SkkBuffer *buf)
 {
 	gchar *preedit;
+	static gint candidate_count = -1;
+	SkkCursor *cursor;
 	DEBUG_DO (printf ("skkbuffer.c : Entering query_dict\n"));
 	if (!buf)
 		return;
 	preedit = create_candidate (buf);
+	cursor = skkbuf_cursor (buf);
 	if (preedit) {
 		set_preedit_buf (buf, preedit);
 		g_free (preedit);
+		if (cursor->caret) {
+			gchar *after = NULL;
+			gint store = cursor->caret;
+			if (separate_string_by_cursor (buf, skkbuf_lasth (buf), NULL, &after)) {
+				if (after) {
+					/* Hack */
+					cursor->caret = 0;
+					cursor->revLen = skkbuf_plen (buf);
+					add_preedit_buf (buf, after);
+					g_free (after);
+					cursor->caret = store;
+				}
+			}
+		}
+		if (candidate_count == -1) {
+			candidate_count = skk_conf_get_num_by_name (skkbuf_conf (buf), "skk-candidate-count");
+		}
+		preedit_emit (buf);
+		if (candidate_count == 0) {
+			skk_buffer_lookup_emit (buf);
+		}
 	} else {
 		set_preedit_buf (buf, "");
 		adddict_emit (buf);
+		preedit_emit (buf);	
 	}
-	preedit_emit (buf);	
 	return;
 }
 
@@ -494,12 +655,27 @@ set_result_buf (SkkBuffer *buf, const gchar *value)
 static void
 set_lasth (SkkBuffer *buf)
 {
+	SkkCursor *cursor;
 	if (!buf)
 		return;
+	cursor = skkbuf_cursor (buf);
 	if (skkbuf_lasth (buf)) {
 		g_free (skkbuf_lasth (buf));
 		skkbuf_lasth (buf) = NULL;
 	}
+#if 0
+	if (cursor && cursor->caret) {
+		gchar *before = NULL;
+		if (separate_string_by_cursor (buf, skkbuf_p (buf), &before, NULL)) {
+			if (before) {
+				skkbuf_lasth (buf) = before;
+			} else {
+				skkbuf_lasth (buf) = g_strdup ("");
+			}
+		}
+		return;
+	}
+#endif
 	if (skkbuf_p (buf)) {
 		skkbuf_lasth (buf) = g_strdup (skkbuf_p (buf));
 	}
@@ -607,6 +783,28 @@ set_buf (SkkBuffer *buf, const gchar *value)
 	return;
 }
 
+static void
+set_numb (SkkBuffer *buf, const gchar *value)
+{
+	if (!buf)
+		return;
+	if (value && skkbuf_numb (buf)) {
+		if (!strcmp (skkbuf_numb (buf), value)) {
+			return;
+		}
+	} else if (!value && !skkbuf_numb (buf)) {
+		return;
+	}
+	if (skkbuf_numb (buf)) {
+		g_free (skkbuf_numb (buf));
+		skkbuf_numb (buf) = NULL;
+	}
+	if (value) {
+		skkbuf_numb (buf) = g_strdup (value);
+	}
+	return;
+}
+
 static gchar*
 get_buf (SkkBuffer *buf, int ch, gboolean lower)
 {
@@ -653,7 +851,7 @@ add_buffer_normal (SkkBuffer *buf, int ch)
 		return FALSE;
 
 	/* Check whether we can translate buf to KANA */
-	buf_is_exist = skkconv_is_exist (tmp_in_buf);
+	buf_is_exist = skk_conv_is_exist (buf->rule, tmp_in_buf);
 
 	if (!buf_is_exist) {
 		/* Can not translate to KANA */
@@ -691,6 +889,12 @@ add_buffer_normal (SkkBuffer *buf, int ch)
 			preedit_emit (buf);
 			return TRUE;
 		}
+		if (!skkbuf_b (buf)) {
+			skkbuf_b (buf) = tmp_in_buf;
+			skkbuf_blen (buf) = strlen (tmp_in_buf);
+			preedit_emit (buf);
+			return TRUE;
+		}
 		buf->in_buf[buf->buf_len - 1] = tolower (ch);
 		skkbuf_blen (buf) = strlen (skkbuf_b (buf));
 		g_free (tmp_in_buf);
@@ -736,7 +940,7 @@ add_buffer_henkan (SkkBuffer *buf, int ch)
 		return FALSE;
 	
 	/* Check whether we can translate buf to KANA */
-	buf_is_exist = skkconv_is_exist (tmp_in_buf);
+	buf_is_exist = skk_conv_is_exist (buf->rule, tmp_in_buf);
 
 	if (!buf_is_exist) {
 		/* FAMAO NOW */
@@ -817,7 +1021,7 @@ add_buffer_okuri (SkkBuffer *buf, int ch)
 		return FALSE;
 
 	/* Check whether we can translate buf to KANA */
-	buf_is_exist = skkconv_is_exist (tmp_in_buf);
+	buf_is_exist = skk_conv_is_exist (buf->rule, tmp_in_buf);
 
 	if (!buf_is_exist) {
 		/* Can not translate to KANA */
@@ -1029,7 +1233,7 @@ add_char_jisx0208_latin (SkkBuffer *buf, int ch)
 {
 	const gchar *result;
 	DEBUG_DO (printf ("skkbuffer.c : Entering add_char_jisx0208_latin\n"));
-	result = skkconv_get_jisx0208_latin (ch);
+	result = skk_conv_get_jisx0208_latin (ch);
 	set_result_buf (buf,result);
 	result_emit (buf);
 	if (result)
@@ -1255,10 +1459,29 @@ static void
 add_preedit_buf (SkkBuffer *buf, const gchar *append)
 {
 	gchar *append_buf;
+	SkkCursor *cursor = skkbuf_cursor (buf);
 	DEBUG_DO (printf ("skkbuffer.c : Entering add_preedit_buf\n"));
-	if (!append) return;
+	if (!append) 
+		return;
 	if (skkbuf_p (buf)) {
-		append_buf = g_strdup_printf ("%s%s", skkbuf_p (buf), append);
+		if (cursor->caret) {
+			gchar *before = NULL;
+			gchar *after = NULL;
+			if (separate_string_by_cursor (buf, skkbuf_p (buf), &before, &after)) {
+				append_buf = g_strdup_printf ("%s%s%s",
+						before ? before : "", 
+						append,
+						after ? after : "");
+				if (before)
+					g_free (before);
+				if (after)
+					g_free (after);
+			} else {
+				append_buf = g_strconcat (skkbuf_p (buf), append, NULL);
+			}
+		} else {
+			append_buf = g_strconcat (skkbuf_p (buf), append, NULL);
+		}
 		g_free (skkbuf_p (buf));
 		skkbuf_p (buf) = append_buf;
 	} else {
@@ -1295,10 +1518,29 @@ add_direction_word (SkkBuffer *buf, const gchar *append)
 {
 	/* FIXME */
 	gchar *append_buf;
+	SkkCursor *cursor = skkbuf_cursor (buf);
 	DEBUG_DO (printf ("skkbuffer.c : Entering add_direction_word\n"));
 	if (!append) return;
 	if (skkbuf_d (buf)) {
-		append_buf = g_strdup_printf ("%s%s", skkbuf_d (buf), append);
+		if (cursor->caret) {
+			/* CURSOR */
+			gchar *before = NULL;
+			gchar *after = NULL;
+			if (separate_string_by_cursor (buf, skkbuf_d (buf), &before, &after)) {
+				append_buf = g_strdup_printf ("%s%s%s",
+						before ? before : "",
+						append,
+						after ? after : "");
+				if (before)
+					g_free (before);
+				if (after)
+					g_free (after);
+			} else {
+				append_buf = g_strconcat (skkbuf_d (buf), append, NULL);
+			}
+		} else {
+			append_buf = g_strconcat (skkbuf_d (buf), append, NULL);
+		}
 		g_free (skkbuf_d (buf));
 		skkbuf_d (buf) = append_buf;
 	} else {
@@ -1317,19 +1559,38 @@ static void
 add_direction_word_int (SkkBuffer *buf, const gchar *append)
 {
 	gchar *append_buf;
+	SkkCursor *cursor = skkbuf_cursor (buf);
 	DEBUG_DO (printf ("skkbuffer.c : Entering add_direction_word_int\n"));
 	if (!append)
 		return;
 	if (!buf->in_number) {
-		if (buf->direction_word) {
-			append_buf = g_strconcat (buf->direction_word,"#",NULL);
+		if (skkbuf_d (buf)) {
+			if (cursor->caret) {
+				gchar *before = NULL;
+				gchar *after = NULL;
+				if (separate_string_by_cursor (buf, skkbuf_d (buf), &before, &after)) {
+					append_buf = g_strdup_printf ("%s%s%s", 
+							before ? before : "",
+							"#",
+							after ? after : "");
+					if (before)
+						g_free (before);
+					if (after)
+						g_free (after);
+				} else {
+					append_buf = g_strconcat (skkbuf_d (buf), "#", NULL);
+				}
+			} else {
+				append_buf = g_strconcat (skkbuf_d (buf),"#",NULL);
+			}
+			g_free (skkbuf_d (buf));
+			skkbuf_d (buf) = append_buf;
 		} else {
-			append_buf = g_strdup ("#");
+			skkbuf_d (buf) = g_strdup ("#");
 		}
 		buf->number_list = g_list_append (buf->number_list,g_strdup (append));
 		buf->has_num++;
 		buf->in_number = TRUE;
-		buf->direction_word = append_buf;
 	} else {
 		GList *last;
 		gchar *last_buf;
@@ -1339,7 +1600,7 @@ add_direction_word_int (SkkBuffer *buf, const gchar *append)
 		last->data = append_buf;
 		g_free (last_buf);
 	}
-	buf->henkan_len = strlen (buf->direction_word);
+	skkbuf_dlen (buf) = strlen (skkbuf_d (buf));
 }
 
 static void
@@ -1407,6 +1668,12 @@ clear (SkkBuffer *buf)
 	if (skkbuf_lasth (buf)) {
 		set_lasth (buf);
 	}
+	if (skkbuf_numb (buf)) {
+		set_numb (buf, NULL);
+	}
+	if (skkbuf_cursor (buf)) {
+		skk_cursor_clear (skkbuf_cursor (buf));
+	}
 	buf->candidate_count = 0;
 	buf->candidate_max = 0;
 	buf->in_completion = FALSE;
@@ -1460,9 +1727,43 @@ translate_okuri (SkkBuffer *buf)
 static gchar*
 get_preedit_string (SkkBuffer *buf)
 {
+	gchar *before = NULL;
+	gchar *after = NULL;
+	gchar *ret = NULL;
+	SkkCursor *cursor = skkbuf_cursor (buf);
 	DEBUG_DO (printf ("skkbuffer.c : Entering get_preedit_string\n"));
-	if (!buf) return NULL;
-	if (skk_conf_get_echo (skkbuf_conf (buf))) {
+	if (!buf)
+		return NULL;
+	if (cursor->caret && skkbuf_p (buf)) {
+		if (separate_string_by_cursor (buf, skkbuf_p (buf), &before, &after)) {
+			if (skk_conf_get_bool_by_name (skkbuf_conf (buf), "skk-echo")) {
+				ret = g_strdup_printf ("%s%s%s%s%s%s",
+				skk_mode_get_query_status_prefix (buf->mode) ?
+				skk_mode_get_query_status_prefix (buf->mode) : "",
+				before ? before : "",
+				skk_mode_get_query_status_append (buf->mode) ?
+				skk_mode_get_query_status_append (buf->mode) : "",
+				skkbuf_okuri (buf) ? skkbuf_okuri (buf) :  "",
+				skkbuf_b (buf) ? skkbuf_b (buf) : "",
+				after ? after : "");
+			} else {
+				ret = g_strdup_printf ("%s%s%s%s%s",
+				skk_mode_get_query_status_prefix (buf->mode) ?
+				skk_mode_get_query_status_prefix (buf->mode) : "",
+				before ? before : "",
+				skk_mode_get_query_status_append (buf->mode) ?
+				skk_mode_get_query_status_append (buf->mode) : "",
+				skkbuf_okuri (buf) ? skkbuf_okuri (buf) :  "",
+				after ? after : "");
+			}
+			if (before)
+				g_free (before);
+			if (after)
+				g_free (after);
+		}
+		return ret;
+	}
+	if (skk_conf_get_bool_by_name (skkbuf_conf (buf), "skk-echo")) {
 		return g_strdup_printf ("%s%s%s%s%s",
 				skk_mode_get_query_status_prefix (buf->mode) ?
 				skk_mode_get_query_status_prefix (buf->mode) : "",
@@ -1485,19 +1786,33 @@ get_preedit_string (SkkBuffer *buf)
 static void
 commit (SkkBuffer *buf, gboolean regist_dict)
 {
+	SkkCursor *cursor;
+	gchar *before = NULL;
+	gchar *after = NULL;
+	gint store = 0;
 	if (!buf)
 		return;
-	if (skkbuf_p (buf)) {
-		set_result_buf (buf, skkbuf_p (buf));
-	}	
+	/* get cursor */
+	cursor = skkbuf_cursor (buf);
+	if (cursor->caret) {
+		/* if we have caret, separate string */
+		separate_string_by_cursor (buf, skkbuf_p (buf), &before, &after);
+		store = cursor->caret;
+	}
+	set_result_buf (buf, before ? before : skkbuf_p (buf));
 	/* TODO */
 	/* add dict */
 	if (regist_dict)
 		skk_buffer_add_dict (buf);
 
+	cursor->caret = 0;
 	if (skkbuf_okuri (buf)) {
 		add_result_buf (buf, skkbuf_okuri (buf));
 	}
+	if (after) {
+		add_result_buf (buf, after);
+	}
+	cursor->caret = store;
 	skk_buffer_set_j_status (buf, SKK_NONE);
 	/* clear unused buffer */
 	set_okurigana (buf, NULL);
@@ -1506,6 +1821,10 @@ commit (SkkBuffer *buf, gboolean regist_dict)
 	set_lasth (buf);
 	set_buf (buf, NULL);
 	set_direction_word (buf, NULL);
+	if (before) 
+		g_free (before);
+	if (after)
+		g_free (after);
 	result_emit (buf);
 }
 
@@ -1547,8 +1866,8 @@ get_translate_buf (SkkBuffer *buf)
 		return NULL;
 	katakana_state = skk_mode_get_katakana (skkbuf_mode (buf));
 	ret = (gchar *) katakana_state ?
-			skkconv_get_katakana (skkbuf_b (buf), &append) :
-			skkconv_get_hiragana (skkbuf_b (buf), &append);
+			skk_conv_get_katakana (buf->rule, skkbuf_b (buf), &append) :
+			skk_conv_get_hiragana (buf->rule, skkbuf_b (buf), &append);
 	if (ret) {
 		set_buf (buf,NULL);
 		if (append) {
@@ -1558,8 +1877,8 @@ get_translate_buf (SkkBuffer *buf)
 	}
 	if (skk_buffer_get_jisx0201_katakana (buf)) {
 		jisx0201 = katakana_state ?
-			skkconv_katakana_to_jisx0201_kana (ret) :
-			skkconv_hiragana_to_jisx0201_kana (ret);
+			skk_conv_katakana_to_jisx0201_kana (ret) :
+			skk_conv_hiragana_to_jisx0201_kana (ret);
 
 		if (jisx0201) {
 			g_free (ret);
@@ -1572,6 +1891,7 @@ get_translate_buf (SkkBuffer *buf)
 static gchar*
 create_candidate (SkkBuffer *buf)
 {
+	gchar *direction = NULL;
 	gchar *okuri = NULL;
 	gchar *kana;
 	DEBUG_DO (printf ("skkbuffer.c : Entering create_candidate\n"));
@@ -1584,13 +1904,13 @@ create_candidate (SkkBuffer *buf)
 		skk_dict_item_destroy_all (buf->candidate_list);
 		buf->candidate_list = NULL;
 	}
-	/* if we have a hankan_okuri_char, use it (SKK_OKURI -> SKK_CHOICE) */
+	/* if we have a henkan_okuri_char, use it (SKK_OKURI -> SKK_CHOICE) */
 	if (skkbuf_okurip (buf)) {
 		okuri = skkbuf_okurip (buf);
 	} else {
 		/* try to translate last buf to char (SKK_HENKAN -> SKK_CHOICE) */
 		if (skkbuf_b (buf)) {
-			if (skkconv_is_exist (skkbuf_b (buf)))  {
+			if (skk_conv_is_exist (buf->rule, skkbuf_b (buf)))  {
 				kana = get_translate_buf (buf);
 				if (kana) {
 					/*  if successfully translated,
@@ -1610,38 +1930,27 @@ create_candidate (SkkBuffer *buf)
 			}
 		}
 	}
+
 	/* store preedit_buf for UNDO */
 	set_lasth (buf);
 
+	direction = get_direction_word (buf);
+
 	/* if we use KATAKANA, translate to HIRAGANA */
-	if (skk_buffer_get_katakana (buf) || skk_buffer_get_jisx0201_katakana (buf)) {
-		gchar *query = NULL;
-		gchar *jisx0201 = NULL;
-		if (skk_mode_get_jisx0201_katakana (skkbuf_mode (buf))) {
-			jisx0201 = skkconv_jisx0201_kana_to_katakana (skkbuf_d (buf));
+	buf->candidate_list = skk_query_do_query (skkbuf_query (buf),
+			direction ? direction : skkbuf_d (buf),
+			okuri, &buf->candidate_max);
+
+	if (!buf->candidate_list) {
+		while (skk_query_set_next (skkbuf_query (buf))) {
+			buf->candidate_list = skk_query_do_query (skkbuf_query (buf),
+				direction ? direction : skkbuf_d (buf),
+				okuri, &buf->candidate_max);
+			if (buf->candidate_list)
+				break;
 		}
-		query = skkconv_katakana_to_hiragana (jisx0201 ? jisx0201 : skkbuf_d (buf));
-		if (jisx0201)
-			g_free (jisx0201);
-		buf->candidate_list = skk_query_do_query (buf->query, query, okuri, &buf->candidate_max);
-		if (!buf->candidate_list) {
-			while (skk_query_set_next (buf->query)) {
-				buf->candidate_list = skk_query_do_query (buf->query, query, okuri, &buf->candidate_max);
-				if (buf->candidate_list)
-					break;
-			}		
-		}
-		g_free (query);
-	} else {
-		buf->candidate_list = skk_query_do_query (buf->query, skkbuf_d (buf), okuri, &buf->candidate_max);
-		if (!buf->candidate_list) {
-			while (skk_query_set_next (buf->query)) {
-				buf->candidate_list = skk_query_do_query (buf->query, skkbuf_d (buf), okuri, &buf->candidate_max);
-				if (buf->candidate_list)
-					break;
-			}
-		}	
 	}
+				
 	DEBUG_DO (printf ("skkbuffer.c : create_candidate buf->candidate_max %d\n", buf->candidate_max));
 	/* Reset current count */
 	buf->candidate_count = 0;
@@ -1651,6 +1960,8 @@ create_candidate (SkkBuffer *buf)
 		query_num (buf);
 	}
 
+	if (direction)
+		g_free (direction);
 	/* return string */
 	if (buf->candidate_list) 
 		return g_strdup (((SkkDictItem *)buf->candidate_list->data)->candidate);
@@ -1695,6 +2006,8 @@ skk_buffer_new (void)
 	ret->mode = skk_mode_new ();
 	ret->conf = NULL;
 	ret->query = NULL;
+	ret->cursor = skk_cursor_new ();
+	ret->rule = NULL;
 	/* set default functions. */
 	ret->clear = clear;
 	ret->set_status = set_status;
@@ -1714,6 +2027,8 @@ skk_buffer_new (void)
 	ret->change_category = change_category;
 	ret->undo = undo;
 	ret->commit = commit;
+	ret->move_left = move_left;
+	ret->move_right = move_right;
 	return ret;
 }
 
@@ -1724,6 +2039,9 @@ skk_buffer_destroy (SkkBuffer *buf)
 	skk_buffer_clear (buf);
 	skk_mode_destroy (buf->mode);
 	skk_conf_destroy (buf->conf);
+	skk_query_destroy (buf->query);
+	skk_conv_rule_destroy (buf->rule);
+	skk_cursor_destroy (skkbuf_cursor (buf));
 	if (buf->preedit_listener) {
 		skk_utils_list_free (buf->preedit_listener, TRUE, NULL, NULL);
 		buf->preedit_listener = NULL;
@@ -1914,17 +2232,23 @@ gboolean
 skk_buffer_has_next_candidate (SkkBuffer *buf)
 {
 	GList *next_candidate;
+	gchar *direction;
 	next_candidate = g_list_next (skkbuf_cur_candidate_list (buf));
 	if (!next_candidate) {
 		GList *append_list = NULL;
+		direction = get_direction_word (buf);
 		while (skk_query_set_next (buf->query)) {
-			append_list = skk_query_do_query (buf->query, skkbuf_d (buf), skkbuf_okurip (buf), NULL);
+			append_list = skk_query_do_query (skkbuf_query (buf), 
+					direction ? direction : skkbuf_d (buf), 
+					skkbuf_okurip (buf), NULL);
 			if (append_list) {
 				buf->candidate_list = skk_dict_item_merge_list (buf->candidate_list, append_list);
 				buf->candidate_max = g_list_length (buf->candidate_list);
 				break;
 			}
 		}
+		if (direction)
+			g_free (direction);
 		if (!buf->candidate_list)
 			return FALSE;
 		next_candidate = g_list_next (skkbuf_cur_candidate_list (buf));
@@ -1940,9 +2264,12 @@ gboolean
 skk_buffer_set_next_candidate (SkkBuffer *buf)
 {
 	gchar *candidate;
+	SkkCursor *cursor;
 	DEBUG_DO (printf ("skkbuffer.c : Entering skk_buffer_set_next_candidate\n"));
 	if (!buf)
 		return FALSE;
+	/* Get Cursor */
+	cursor = skkbuf_cursor (buf);
 	if (!buf->candidate_list) {
 		if (!skk_buffer_has_next_candidate (buf)) {
 			return FALSE;
@@ -1955,6 +2282,20 @@ skk_buffer_set_next_candidate (SkkBuffer *buf)
 	candidate = skkbuf_cur_candidate (buf);
 	if (candidate) {
 		set_preedit_buf (buf,candidate);
+		if (cursor->caret) {
+			gchar *after = NULL;
+			gint store = cursor->caret;
+			if (separate_string_by_cursor (buf, skkbuf_lasth (buf), NULL, &after)) {
+				if (after) {
+					/* Hack */
+					cursor->caret = 0;
+					cursor->revLen = skkbuf_plen (buf);
+					add_preedit_buf (buf, after);
+					g_free (after);
+					cursor->caret = store;
+				}
+			}
+		}
 		preedit_emit (buf);
 		return TRUE;
 	}
@@ -1965,6 +2306,7 @@ gboolean
 skk_buffer_set_prev_candidate (SkkBuffer *buf)
 {
 	gchar *candidate;
+	SkkCursor *cursor;
 	DEBUG_DO (printf ("skkbuffer.c : Entering skk_buffer_set_prev_candidate\n"));
 	if (!buf)
 		return FALSE;
@@ -1972,15 +2314,30 @@ skk_buffer_set_prev_candidate (SkkBuffer *buf)
 		return FALSE;
 	if (buf->candidate_count < 1)
 		return FALSE;
+	cursor = skkbuf_cursor (buf);
 	buf->candidate_count--;
 	candidate = skkbuf_cur_candidate (buf);
 	if (!candidate)
 		return FALSE;
 	if (candidate) {
 		set_preedit_buf (buf,candidate);
+		if (cursor->caret) {
+			gchar *after = NULL;
+			gint store = cursor->caret;
+			if (separate_string_by_cursor (buf, skkbuf_lasth (buf), NULL, &after)) {
+				if (after) {
+					cursor->caret = 0;
+					cursor->revLen = skkbuf_plen (buf);
+					add_preedit_buf (buf, after);
+					g_free (after);
+					cursor->caret = store;
+				}
+			}
+		}
 		preedit_emit (buf);
 		return TRUE;
 	}
+	return FALSE;
 }
 
 gchar *
@@ -2293,25 +2650,65 @@ skk_buffer_set_query (SkkBuffer *buf, SkkQuery *query)
 	if (!buf)
 		return;
 	if (buf->query) {
-		skk_query_destroy (buf->query);
-		buf->query = NULL;
+		skk_query_unref (buf->query);
 	}
 	buf->query = query;
 	skk_query_ref (buf->query);
+	skk_query_set_conf (buf->query, buf->conf);
 	return;
 }
 
+void
+skk_buffer_set_rule (SkkBuffer *buf, SkkConvRule *rule)
+{
+	if (!buf)
+		return;
+	if (buf->rule) {
+		skk_conv_rule_unref (buf->rule);
+	}
+	buf->rule = rule;
+	skk_conv_rule_ref (buf->rule);
+	/* TODO */
+	
+}
+
 void
 skk_buffer_set_conf (SkkBuffer *buf, SkkConf *conf)
 {
+	gchar *status_str;
 	if (!buf)
 		return;
 	if (buf->conf) {
-		skk_conf_destroy (buf->conf);
-		buf->conf = NULL;
+		skk_conf_unref (buf->conf);
 	}
 	buf->conf = conf;
 	skk_conf_ref (buf->conf);
+	skk_query_set_conf (buf->query, buf->conf);
+	if ((status_str = skk_conf_get_string_by_name (skkbuf_conf (buf), "skk-abbrev-mode-string"))) {
+		skk_mode_set_abbrev_str (skkbuf_mode (buf), status_str);
+		g_free (status_str);
+	}
+	if ((status_str = skk_conf_get_string_by_name (skkbuf_conf (buf), "skk-hirakana-mode-string"))) {
+		skk_mode_set_hirakana_str (skkbuf_mode (buf), status_str);
+		g_free (status_str);
+	}
+	if ((status_str = skk_conf_get_string_by_name (skkbuf_conf (buf), "skk-katakana-mode-string"))) {
+		skk_mode_set_katakana_str (skkbuf_mode (buf), status_str);
+		g_free (status_str);
+	}
+	if ((status_str = skk_conf_get_string_by_name (skkbuf_conf (buf), "skk-jisx0201-kana-mode-string"))) {
+		skk_mode_set_jisx0201_kana_str (skkbuf_mode (buf), status_str);
+		g_free (status_str);
+	}
+	if ((status_str = skk_conf_get_string_by_name (skkbuf_conf (buf), "skk-jisx0208-latin-mode-string"))) {
+		skk_mode_set_jisx0208_latin_str (skkbuf_mode (buf), status_str);
+		g_free (status_str);
+	}
+	if ((status_str = skk_conf_get_string_by_name (skkbuf_conf (buf), "skk-latin-mode-string"))) {
+		skk_mode_set_latin_str (skkbuf_mode (buf), status_str);
+		g_free (status_str);
+	}
+
 	return;
 }
 
@@ -2334,6 +2731,8 @@ skk_buffer_add_lookup_listener (SkkBuffer *buf, SkkLookupListener listener, gpoi
 	SkkLookupFunc *func;
 	if (!buf)
 		return;
+	if (!listener)
+		return;
 	func = g_new (SkkLookupFunc, 1);
 	func->listener = listener;
 	func->user_data = user_data;
@@ -2347,6 +2746,8 @@ skk_buffer_add_adddict_listener (SkkBuffer *buf, SkkAddDictListener listener, gp
 	SkkAddDictFunc *func;
 	if (!buf)
 		return;
+	if (!listener)
+		return;
 	func = g_new (SkkAddDictFunc, 1);
 	func->listener = listener;
 	func->user_data = user_data;
@@ -2360,6 +2761,9 @@ skk_buffer_add_preedit_listener (SkkBuffer *buf, SkkBufListener listener, gpoint
 	SkkBufFunc *func;
 	if (!buf)
 		return;
+	if (!listener) {
+		return;
+	}
 	func = g_new (SkkBufFunc, 1);
 	func->listener = listener;
 	func->user_data = user_data;
@@ -2373,6 +2777,8 @@ skk_buffer_add_result_listener (SkkBuffer *buf, SkkBufListener listener, gpointe
 	SkkBufFunc *func;
 	if (!buf)
 		return;
+	if (!listener)
+		return;
 	func = g_new (SkkBufFunc, 1);
 	func->listener = listener;
 	func->user_data = user_data;
@@ -2413,14 +2819,26 @@ skk_buffer_remove_codetable_listener (SkkBuffer *buf, SkkCodeTableListener liste
 void
 skk_buffer_add_dict (SkkBuffer *buf)
 {
+	gchar *direction;
 	if (!buf)
 		return;
 	if (!skkbuf_r (buf))
 		return;
 	if (!skkbuf_d (buf))
 		return;
-	skk_query_set_with_type (buf->query, SKKQUERY_LOCAL);
-	skk_query_add (buf->query, skkbuf_d (buf), skkbuf_okurip (buf), skkbuf_r (buf));
+	/* FIXME not handle in the case of buf->has_num > 0) */
+	/* rewind search pligin */
+	skk_query_set_first (skkbuf_query (buf));
+	skk_query_set_with_type (skkbuf_query (buf), SKKQUERY_LOCAL);
+	direction = get_direction_word (buf);
+	skk_query_add (skkbuf_query (buf),
+			direction ? direction : skkbuf_d (buf),
+			skkbuf_okurip (buf),
+			buf->has_num ? skkbuf_numb (buf) : skkbuf_r (buf));
+	/* reset */
+	skk_query_set_first (skkbuf_query (buf));
+	if (direction)
+		g_free (direction);
 	return;
 }
 
@@ -2433,8 +2851,12 @@ skk_buffer_add_dict_with_value (SkkBuffer *buf, gchar *value)
 		return;
 	if (!skkbuf_d (buf))
 		return;
-	skk_query_set_with_type (buf->query, SKKQUERY_LOCAL);
+	/* rewind search plugin */
+	skk_query_set_first (skkbuf_query (buf));
+	skk_query_set_with_type (skkbuf_query (buf), SKKQUERY_LOCAL);
 	skk_query_add (buf->query, skkbuf_d (buf), skkbuf_okurip (buf), value);
+	/* reset */
+	skk_query_set_first (skkbuf_query (buf));
 	return;
 }
 
@@ -2458,7 +2880,7 @@ skk_buffer_get_completion (SkkBuffer *buf)
 	while (skk_query_set_next (buf->query)) {
 		append = skk_query_do_completion (buf->query, skkbuf_d (buf));
 		if (append)
-			buf->completion_list = skk_utils_list_merge_string (buf->completion_list, append);
+			buf->completion_list = skk_utils_list_merge_string (buf->completion_list, append, TRUE);
 	}
 	/* re-initialized */
 	skk_query_set_first (buf->query);
@@ -2597,3 +3019,25 @@ skk_buffer_add_result (SkkBuffer *buf, const gchar *value)
 	result_emit (buf);
 	return;
 }
+
+void
+skk_buffer_move_left (SkkBuffer *buf)
+{
+	if (!buf)
+		return;
+	if (buf->move_left) {
+		buf->move_left (buf);
+	}
+	return;
+}
+
+void
+skk_buffer_move_right (SkkBuffer *buf)
+{
+	if (!buf)
+		return;
+	if (buf->move_right) {
+		buf->move_right (buf);
+	}
+	return;
+}
