@@ -32,7 +32,7 @@
  *
  */
 
-/* $Id: skkbuffer.h,v 1.4.2.1 2002/04/15 21:29:56 famao Exp $ */
+/* $Id: skkbuffer.h,v 1.8 2002/11/09 16:56:48 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -40,10 +40,12 @@
 #ifndef HAVE_SKKBUF_H
 #define HAVE_SKKBUF_H
 
-#include "skkmode.h"
-#include "skkconf.h"
-#include "skkquery.h"
-#include "skkdictitem.h"
+#include <skkmode.h>
+#include <skkconf.h>
+#include <skkquery.h>
+#include <skkcursor.h>
+#include <skkconv_kana.h>
+#include <skkdictitem.h>
 
 /* in_buf */
 #define skkbuf_b(x) (x)->in_buf
@@ -62,6 +64,8 @@
 #define skkbuf_okuri(x) (x)->henkan_okurigana
 /* last kakutei */
 #define skkbuf_lasth(x) (x)->last_kakutei_henkan_key
+/* number */
+#define skkbuf_numb(x) (x)->num_buf
 
 /* candidate */
 #define skkbuf_candidate_num(x)		(x)->candidate_count
@@ -82,6 +86,8 @@
 #define skkbuf_mode(x) (x)->mode
 /* query */
 #define skkbuf_query(x) (x)->query
+/* cursor */
+#define skkbuf_cursor(x) (x)->cursor
 
 /* completion */
 #define skkbuf_is_completion(x) (x)->in_completion
@@ -99,6 +105,7 @@ struct _SkkBuffer
 	gchar *in_buf;
 	gchar *preedit_buf;
 	gchar *direction_word;
+	gchar *num_buf;
 	/* Ѵ٤겾̾ʬΥץեå */
 	gchar *henkan_okuri_char;
 	/* ߤѴ겾̾ʬ */
@@ -127,6 +134,8 @@ struct _SkkBuffer
 	SkkMode *mode;
 	SkkConf *conf;
 	SkkQuery *query;
+	SkkCursor *cursor;
+	SkkConvRule *rule;
 
 	/* listener */
 	GList *preedit_listener;
@@ -167,6 +176,9 @@ struct _SkkBuffer
 	void           (*change_category) (SkkBuffer *buf);
 	/* Commit String */
 	void           (*commit) (SkkBuffer *buf, gboolean regist_dict);
+	/* Cursor */
+	void           (*move_left) (SkkBuffer *buf);
+	void           (*move_right) (SkkBuffer *buf);
 
 };
 
@@ -247,6 +259,9 @@ void skk_buffer_set_conf (SkkBuffer *buf, SkkConf *conf);
 /* set query */
 void skk_buffer_set_query (SkkBuffer *buf, SkkQuery *query);
 
+/* set conversion rule */
+void skk_buffer_set_rule (SkkBuffer *buf, SkkConvRule *rule);
+
 /* Event Listener */
 void skk_buffer_add_preedit_listener (SkkBuffer *buf, SkkBufListener listener, gpointer user_data);
 void skk_buffer_add_result_listener (SkkBuffer *buf, SkkBufListener listener, gpointer user_data);
@@ -272,4 +287,8 @@ void skk_buffer_add_preedit (SkkBuffer *buf, const gchar *value);
 void skk_buffer_set_result (SkkBuffer *buf, const gchar *value);
 void skk_buffer_add_result (SkkBuffer *buf, const gchar *value);
 
+/* Cursor */
+void skk_buffer_move_left (SkkBuffer *buf);
+void skk_buffer_move_right (SkkBuffer *buf);
+
 #endif /* HAVE_SKKBUF_H */
