@@ -32,7 +32,7 @@
  *
  */
 
-/* $Id: skkcomm.c,v 1.4 2002/03/15 23:00:49 famao Exp $ */
+/* $Id: skkcomm.c,v 1.7 2002/11/09 16:56:48 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -45,14 +45,21 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
+#include <errno.h>
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "skkcomm.h"
 #include "skkdebug.h"
 
-/* FIXME */
-static int recent_cmd = SKKSERV_BYE;
 
+#ifdef HAVE_GETADDRINFO
 gint
-skk_comm_makesock (gchar * host, gchar * service)
+skk_comm_makesock (const gchar * host, const gchar * service)
 {
   struct addrinfo hints, *res, *res0;
   int sock = -1;
@@ -68,6 +75,7 @@ skk_comm_makesock (gchar * host, gchar * service)
 		  continue;
       if (connect (sock, (struct sockaddr *) res->ai_addr, res->ai_addrlen) < 0) {
 		  close (sock);
+		  sock = -1;
 		  continue;
 	  }
       break;
@@ -75,68 +83,110 @@ skk_comm_makesock (gchar * host, gchar * service)
   freeaddrinfo (res0);
   return sock;
 }
+#else
+gint
+skk_comm_makesock (const gchar *host, const gchar *service)
+{
+	/* FIXME error check */
+	struct sockaddr_in addr;
+	struct hostent *he;
+	struct servent *se;
+	int sock = -1;
+	if (!host || !service) {
+		return -1;
+	}
+	memset (&addr, 0, sizeof (addr));
+	sock = socket (PF_INET, SOCK_STREAM, 0);
+	if (sock < 0)
+		return -1;
+	he = gethostbyname (host);
+	if (!he) {
+		return -1;
+	}
+	se = getservbyname (service, "tcp");
+	if (!se) {
+		return -1;
+	}
+	addr.sin_family = PF_INET;
+	addr.sin_port = se->s_port;
+	memcpy (&(addr.sin_addr), he->h_addr,
+			he->h_length);
+	if (connect (sock, (struct sockaddr *) &addr, sizeof (struct sockaddr_in)) < 0) {
+		close (sock);
+		sock = -1;
+	}
+	return sock;
+}
+#endif
 
 gboolean
-skk_comm_send (gint sock, SkkServInfo info, gchar * string)
+skk_comm_send (gint sock, SkkServInfo info, const gchar *string)
 {
-	/* store recent status */
-	recent_cmd = info;
-  switch (info)
-    {
-    case SKKSERV_BYE:
-    case SKKSERV_VERSION:
-    case SKKSERV_HOSTNAME:
-      {
-	gchar *push = g_strdup_printf ("%d\n", info);
-	send (sock, (void *) push, strlen (push), 0);
-	g_free (push);
-      }
-      break;
-    case SKKSERV_SEARCH:
-	  {
-		  gchar *push = g_strdup_printf ("1%s \n",string);
-	      send (sock, (void *) push, strlen (push), 0);
-		  g_free (push);
-	  }
-      break;
-    default:
-      return FALSE;
-      break;
-    }
-  return TRUE;
+	gint saved_errno = 0;
+
+	switch (info) {
+		case SKKSERV_BYE:
+		case SKKSERV_VERSION:
+		case SKKSERV_HOSTNAME:
+			{
+				gchar *push = g_strdup_printf ("%d\n", info);
+				/* block SIGPIPE */
+				send (sock, (void *) push, strlen (push), MSG_NOSIGNAL);
+				saved_errno = errno;
+				g_free (push);
+			}
+			break;
+		case SKKSERV_SEARCH:
+			{
+				gchar *push = NULL;
+				if (!string)
+					return FALSE;
+				push = g_strdup_printf ("1%s \n", string);
+				/* block SIGPIPE */
+				send (sock, (void *) push, strlen (push), MSG_NOSIGNAL);
+				saved_errno = errno;
+				g_free (push);
+			}
+			break;
+		default:
+			return FALSE;
+			break;
+	}
+	if (saved_errno == EPIPE)
+		return FALSE;
+	return TRUE;
 }
 
 gboolean
-skk_comm_recv (gint sock, gchar ** answer,gint *recv_status)
+skk_comm_recv (gint sock, SkkServInfo info, gchar ** answer,gint *recv_status)
 {
   gint len = BUFSIZ;
   gint retlen;
-  guchar status;
+  guchar status = 0;
   gchar *val;
+  gchar *sep;
 
   /* first, get status */
-  if (recent_cmd == SKKSERV_SEARCH )
+  if (info == SKKSERV_SEARCH )
 	  retlen = recv (sock, (void *) &status, 1, 0);
-  val = g_malloc (len);
+  val = g_new (gchar, len);
 
   while (TRUE) {
 	  if ((retlen = recv (sock, (void *) val, len, MSG_PEEK)) < len)
 	  {
-		  gchar *tmp;
-		  tmp = strchr (val, '\n');
-		  if (tmp) {
-			  retlen = tmp - val + 1;
-		  }
-		  g_realloc (val, retlen);
+		  val = g_realloc (val, retlen + 1);
 		  break;
 	  }
       len = len * 2;
       val = g_realloc (val, len);
   }
   recv (sock, (void *) val, retlen, 0);
-  val[retlen - 1] = '\0';
+  val[retlen] = '\0';
+  if ((sep = strrchr (val, '\n'))) {
+	  *sep = '\0';
+  }
 #ifdef SKKCOMM_DEBUG
-  g_message ("skk_comm_recv \tval %s#", val);
+  g_message ("skk_comm_recv strlen %d retlen %d \tval %s#", strlen (val), retlen, val);
 #endif
   if (answer)
     *answer = val;
@@ -162,12 +212,16 @@ main (gint argc, gchar ** argv)
   gchar *sp;
   int c;
   sock = skk_comm_makesock ("localhost", "skkserv");
+  if (sock < 0) {
+  	g_message ("cannot connect");
+	return 1;
+  }	
   skk_comm_send (sock, SKKSERV_SEARCH, "j");
-  skk_comm_recv (sock, &mes, &status);
+  skk_comm_recv (sock, SKKSERV_SEARCH, &mes, &status);
   if (mes)
 	  printf ("result :%s\n", mes);
   skk_comm_send (sock, SKKSERV_SEARCH, "");
-  skk_comm_recv (sock, &mes, &status);
+  skk_comm_recv (sock, SKKSERV_SEARCH, &mes, &status);
   if (mes)
 	  printf ("result :%s\n", mes);
 
@@ -178,17 +232,17 @@ main (gint argc, gchar ** argv)
 	  printf ("%s\n", sp);
   }
   skk_comm_send (sock, SKKSERV_SEARCH, "k");
-  skk_comm_recv (sock, &mes, &status);
+  skk_comm_recv (sock, SKKSERV_SEARCH, &mes, &status);
   if (mes)
 	  printf ("result :%s\n", mes);
   skk_comm_send (sock, SKKSERV_SEARCH, "#m");
-  skk_comm_recv (sock, &mes ,&status);
+  skk_comm_recv (sock, SKKSERV_SEARCH, &mes ,&status);
   if (mes)
 	  printf ("result :%s\n", mes);
   skk_comm_hostname (sock);
-  skk_comm_recv (sock, NULL,NULL);
+  skk_comm_recv (sock, SKKSERV_HOSTNAME, NULL,NULL);
   skk_comm_version (sock);
-  skk_comm_recv (sock, NULL,NULL);
+  skk_comm_recv (sock, SKKSERV_VERSION, NULL,NULL);
   shutdown (sock, 2);
   close (sock);
   return 0;
