@@ -32,7 +32,7 @@
  *
  */
 
-/* $Id: skkcomm.h,v 1.2 2002/03/15 21:32:40 famao Exp $ */
+/* $Id: skkcomm.h,v 1.3 2002/11/09 16:56:48 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -45,15 +45,15 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/socket.h>
-#include "skktypes.h"
+#include <skktypes.h>
 
 #define skk_comm_bye(x)		skk_comm_send (x, SKKSERV_BYE, NULL)
 #define skk_comm_version(x)	skk_comm_send (x, SKKSERV_VERSION, NULL)
 #define skk_comm_hostname(x) skk_comm_send (x, SKKSERV_HOSTNAME, NULL)
 
-gint skk_comm_makesock (gchar *host, gchar *service);
-gboolean skk_comm_send (gint sock, SkkServInfo info, gchar *string);
-gboolean skk_comm_recv (gint sock, gchar **answer, gint *recv_status);
+gint skk_comm_makesock (const gchar *host, const gchar *service);
+gboolean skk_comm_send (gint sock, SkkServInfo info, const gchar *string);
+gboolean skk_comm_recv (gint sock, SkkServInfo info, gchar **answer, gint *recv_status);
 
 
 #endif
