@@ -32,7 +32,7 @@
  *
  */
 
-/* $Id: skkconf.h,v 1.2 2002/04/09 20:42:53 famao Exp $ */
+/* $Id: skkconf.h,v 1.4 2002/11/09 16:56:48 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -40,27 +40,32 @@
 #ifndef HAVE_SKKCONF_H
 #define HAVE_SKKCONF_H
 #include <glib.h>
-
-#define skkconf_latinstr(x) (x)->skk_latin_mode_string
-#define skkconf_hirastr(x) (x)->skk_hirakana_mode_string
-#define skkconf_katastr(x) (x)->skk_katakana_mode_string
-#define skkconf_jisx0208str(x) (x)->skk_jisx0208_latin_mode_string
+#include <skktypes.h>
 
 typedef struct _SkkConf SkkConf;
+typedef struct _SkkConfItem SkkConfItem;
+
+typedef void (*SkkConfRuleListener) (SkkConf *conf, const gchar *key, const gchar *hira, 
+		const gchar *kana, const gchar *append, gpointer user_data);
+
+struct _SkkConfItem {
+	SkkConfType type;
+	SkkConfItemType itemtype;
+	gchar *name;
+	gchar *info;
+	gpointer value;
+	gboolean user_defined;
+};
 
 struct _SkkConf {
 	gint ref_count;
-	gint skk_number_style;
-	gchar *skk_latin_mode_string;
-	gchar *skk_hirakana_mode_string;
-	gchar *skk_katakana_mode_string;
-	gchar *skk_jisx0208_latin_mode_string;
-	gboolean skk_echo;
-	gboolean skk_egg_like_newline;
-	gboolean skk_delete_implies_kakutei;
-	gboolean skk_delete_okuri_when_quit;
-	gboolean skk_dabbrev_like_completion;
-	gboolean skk_date_ad;
+
+	GList *conf_bool;
+	GList *conf_string;
+	GList *conf_num;
+
+	/* Event Listener */
+	GList *conf_rule_listener;
 
 	void (*clear) (SkkConf *conf);
 };
@@ -71,26 +76,26 @@ void skk_conf_clear (SkkConf *conf);
 void skk_conf_ref (SkkConf *conf);
 void skk_conf_unref (SkkConf *conf);
 
-/* egg_like_newline */
-gboolean skk_conf_get_egg_like_newline (SkkConf *conf);
-void skk_conf_set_egg_like_newline (SkkConf *conf, gboolean state);
-/* echo */
-gboolean skk_conf_get_echo (SkkConf *conf);
-void skk_conf_set_echo (SkkConf *conf, gboolean state);
-/* delete_implies_kakutei */
-gboolean skk_conf_get_delete_implies_kakutei (SkkConf *conf);
-void skk_conf_set_delete_implies_kakutei (SkkConf *conf, gboolean state);
-/* delete_okuri_when_quit */
-gboolean skk_conf_get_delete_okuri_when_quit (SkkConf *conf);
-void skk_conf_set_delete_okuri_when_quit (SkkConf *conf, gboolean state);
-/* dabbrev_like_completion */
-gboolean skk_conf_get_dabbrev_like_completion (SkkConf *conf);
-void skk_conf_set_dabbrev_like_completion (SkkConf *conf, gboolean state);
-/* skk_date_ad */
-gboolean skk_conf_get_date_ad (SkkConf *conf);
-void skk_conf_set_date_ad (SkkConf *conf, gboolean state);
-/* skk_number_style */
-gint skk_conf_get_number_style (SkkConf *conf);
-void skk_conf_set_number_style (SkkConf *conf, gint style);
+SkkConfItem* skk_conf_item_new (void);
+SkkConfItem* skk_conf_item_new_with_value (SkkConfType type, SkkConfItemType itemtype, const gchar *name, const gchar *info, gpointer value);
+void skk_conf_item_destroy (SkkConfItem *item);
+void skk_conf_add_item (SkkConf *conf, SkkConfItem *item);
 
+typedef enum {
+	SKKCONF_CONFIG_ITEM=0,
+	SKKCONF_CONFIG_RULE,
+	SKKCONF_CONFIG_INVALID,
+} SkkConfConfigType;
+
+void skk_conf_load_rc(SkkConf *conf, char *user_name);
+
+gboolean skk_conf_get_bool (SkkConf *conf, SkkConfItemType type);
+gchar *skk_conf_get_string (SkkConf *conf, SkkConfItemType type);
+gint skk_conf_get_num (SkkConf *conf, SkkConfItemType type);
+
+gboolean skk_conf_get_bool_by_name (SkkConf *conf, const gchar *name);
+gchar *skk_conf_get_string_by_name (SkkConf *conf, const gchar *name);
+gint skk_conf_get_num_by_name (SkkConf *conf, const gchar *name);
+
+gboolean skk_conf_add_rule_listener (SkkConf *conf, SkkConfRuleListener listener, gpointer user_data);
 #endif /* HAVE_SKKCONF_H */
