@@ -32,7 +32,7 @@
  *
  */
 
-/* $Id: skkconv_jisx0201_kana.c,v 1.4.2.5 2002/04/15 00:08:10 famao Exp $ */
+/* $Id: skkconv_jisx0201_kana.c,v 1.12 2002/11/09 16:56:49 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -52,7 +52,12 @@ translate_to_jisx0201_kana (const gchar *str)
 	if (!str)
 		return NULL;
 	for (i = 0; i < SKKCONV_JISX0201_SIZE ; i++) {
+#if 1
+		/* Katakana is always 2bytes */
+		if (!strncmp (jisx0201_list[i].katakana , str, 2)) {
+#else
 		if (!strncmp (jisx0201_list[i].katakana , str, strlen (jisx0201_list[i].katakana))) {
+#endif
 			return (jisx0201_list[i]).jisx0201_kana;
 		}
 	}
@@ -60,7 +65,7 @@ translate_to_jisx0201_kana (const gchar *str)
 }
 
 gchar *
-skkconv_katakana_to_jisx0201_kana (const gchar *katakana)
+skk_conv_katakana_to_jisx0201_kana (const gchar *katakana)
 {
 	gchar *ret, *buf;
 	const gchar *trans;
@@ -83,21 +88,21 @@ skkconv_katakana_to_jisx0201_kana (const gchar *katakana)
 		}
 		katakana += len;
 	}
+	ret = g_realloc (ret, actual_len + 1);
 	ret[actual_len] = '\0';
-	ret = g_realloc (ret, actual_len);
 	return ret;
 }
 
 gchar*
-skkconv_hiragana_to_jisx0201_kana (const gchar *hiragana)
+skk_conv_hiragana_to_jisx0201_kana (const gchar *hiragana)
 {
 	/* TODO */
 	gchar *katakana;
 	gchar *ret;
 	if (!hiragana)
 		return NULL;
-	katakana = skkconv_hiragana_to_katakana (hiragana);
-	ret = skkconv_katakana_to_jisx0201_kana (katakana);
+	katakana = skk_conv_hiragana_to_katakana (hiragana);
+	ret = skk_conv_katakana_to_jisx0201_kana (katakana);
 	g_free (katakana);
 	return ret;
 }
@@ -109,7 +114,7 @@ find_kana (gconstpointer p1, gconstpointer p2)
 }
 
 gchar*
-skkconv_jisx0201_kana_to_katakana (const gchar *jisx0201_kana)
+skk_conv_jisx0201_kana_to_katakana (const gchar *jisx0201_kana)
 {
 	jisx0201_rule_list *list;
 	const gchar *copy;
@@ -161,14 +166,17 @@ redo:
 int
 main (void)
 {
-	printf ("from Hiragana %s\n", skkconv_hiragana_to_jisx0201_kana ("1"));
-	printf ("from Katakana %s\n", skkconv_katakana_to_jisx0201_kana (""));
-	printf ("from Katakana %s\n", skkconv_katakana_to_jisx0201_kana ("Хӥ֥٥ܥѥԥץڥ"));
-	printf ("from Katakana %s\n", skkconv_katakana_to_jisx0201_kana (""));
-	printf ("a %s\n", skkconv_jisx0201_kana_to_katakana ("1"));
-	printf ("a %s\n", skkconv_jisx0201_kana_to_katakana ("ގގގގގʎߎˎߎ̎ߎ͎ߎΎ"));
-	printf ("a %s\n", skkconv_jisx0201_kana_to_katakana ("ގގގގގ"));
-	printf ("from Katakana %s\n", skkconv_katakana_to_jisx0201_kana (""));
+	int i = 0;
+	for (; i < 10000; i++) {
+		printf ("from Hiragana %s\n", skk_conv_hiragana_to_jisx0201_kana ("1"));
+		printf ("from Katakana %s\n", skk_conv_katakana_to_jisx0201_kana (""));
+		printf ("from Katakana %s\n", skk_conv_katakana_to_jisx0201_kana ("Хӥ֥٥ܥѥԥץڥ"));
+		printf ("from Katakana %s\n", skk_conv_katakana_to_jisx0201_kana (""));
+		printf ("a %s\n", skk_conv_jisx0201_kana_to_katakana ("1"));
+		printf ("a %s\n", skk_conv_jisx0201_kana_to_katakana ("ގގގގގʎߎˎߎ̎ߎ͎ߎΎ"));
+		printf ("a %s\n", skk_conv_jisx0201_kana_to_katakana ("ގގގގގ"));
+		printf ("from Katakana %s\n", skk_conv_katakana_to_jisx0201_kana (""));
+	}
 	return 0;
 }
 #endif
