@@ -32,7 +32,7 @@
  *
  */
 
-/* $Id: $ */
+/* $Id: skkconv_jisx0201_kana.h,v 1.4 2002/11/09 16:56:49 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -54,8 +54,8 @@ extern jisx0201_rule_list jisx0201_list[];
 
 #define SKKCONV_JISX0201_SIZE (((sizeof (jisx0201_list)) / (sizeof (jisx0201_list[0]))))
 
-gchar *skkconv_hiragana_to_jisx0201_kana (const gchar *hiragana);
-gchar *skkconv_katakana_to_jisx0201_kana (const gchar *katakana);
-gchar *skkconv_jisx0201_kana_to_katakana (const gchar *jisx0201);
+gchar *skk_conv_hiragana_to_jisx0201_kana (const gchar *hiragana);
+gchar *skk_conv_katakana_to_jisx0201_kana (const gchar *katakana);
+gchar *skk_conv_jisx0201_kana_to_katakana (const gchar *jisx0201);
 
 #endif /* HAVE_SKKCONV_JISX0201_KANA_H */
