@@ -32,7 +32,7 @@
  *
  */
 
-/* $Id: $ */
+/* $Id: skkconv_jisx0201_kana_private.h,v 1.4 2002/11/09 16:56:49 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -40,7 +40,7 @@
 #ifndef SKKCONV_JISX0201_KANA_PRIVATE_H
 #define SKKCONV_JISX0201_KANA_PRIVATE_H
 #include <glib.h>
-#include "skkconv_jisx0201_kana.h"
+#include <skkconv_jisx0201_kana.h>
 
 jisx0201_rule_list jisx0201_list[] = {
 	{	"",	""	},
