@@ -32,52 +32,56 @@
  *
  */
 
-/* $Id: skkconv_jisx0208_latin.c,v 1.3 2002/03/15 21:32:40 famao Exp $ */
+/* $Id: skkconv_jisx0208_latin.c,v 1.7 2002/11/09 16:56:49 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
 
 #include <stdio.h>
 #include <glib.h>
+#include <string.h>
+#include "skkutils.h"
 #include "skkconv_jisx0208_latin_private.h"
 #include "skkconv_jisx0208_latin.h"
 
 const gchar *
-skkconv_get_jisx0208_latin (gint ch)
+skk_conv_get_jisx0208_latin (gint ch)
 {
 	if (ch < 0 || ch > 128) return NULL;
 	return skk_jisx0208_latin_vector[ch];
 }
 
 gchar *
-skkconv_get_jisx0208_latin_from_str (const gchar *str, gboolean store)
+skk_conv_get_jisx0208_latin_from_str (const gchar *str, gboolean store)
 {
-	gchar *ret = NULL;
-	gchar *tmp_result;
+	gint len;
+	gint actual_len = 0;
+	gchar *buf,*ret;
 	const gchar *tmp_buf;
 	const guchar *p;
-	for (p = str; p && *p; p++) {
-		tmp_buf = skkconv_get_jisx0208_latin (*p);
+
+	if (!str)
+		return NULL;
+	ret = buf = g_new (char, (strlen (str) * 2) + 1);
+	p = (const guchar *)str;
+	while (p && *p) {
+		len = skk_utils_charbytes (p);
+		tmp_buf = skk_conv_get_jisx0208_latin (*p);
 		if (!tmp_buf) {
 			if (store) {
-				if (!ret) {
-					ret = g_strdup_printf ("%c",*p);
-				} else {
-					tmp_result = ret;
-					ret = g_strdup_printf ("%s%c", ret, *p);
-					g_free (tmp_result);
-				}
+				strncpy (buf, p, len);
+				buf += len;
+				actual_len += len;
 			}
-			continue;
-		}
-		if (!ret) {
-			ret = g_strdup (tmp_buf);
 		} else {
-			tmp_result = ret;
-			ret = g_strconcat (ret, tmp_buf, NULL);
-			g_free (tmp_result);
+			strcpy (buf, tmp_buf);
+			actual_len += strlen (tmp_buf);
+			buf += strlen (tmp_buf);
 		}
+		p += len;
 	}
+	ret = g_realloc (ret, actual_len + 1);
+	ret[actual_len] = '\0';
 	return ret;
 }
 
@@ -88,14 +92,15 @@ main(void)
 	int ch;
 	const gchar *str;
 	for (ch = 0; ch < 128; ch++) {
-		str = skkconv_get_jisx0208_latin (ch);
+		str = skk_conv_get_jisx0208_latin (ch);
 		if (str)
 			printf ("%d (%c:%s)\t",ch,ch,str);
 		if (ch > 31 && !(ch % 8))
 			printf ("\n");
 	}
 	printf ("\n");
-	printf ("%s\n", skkconv_get_jisx0208_latin_from_str ("ʿ14ǯ", TRUE));
+	printf ("%s\n", skk_conv_get_jisx0208_latin_from_str ("ʿ14ǯ", TRUE));
+	printf ("%s\n", skk_conv_get_jisx0208_latin_from_str ("ʿ14ǯ", FALSE));
 	return 0;
 }
 #endif
