@@ -32,7 +32,7 @@
  *
  */
 
-/* $Id: skkconv_jisx0208_latin.h,v 1.3 2002/03/15 21:32:40 famao Exp $ */
+/* $Id: skkconv_jisx0208_latin.h,v 1.3 2002/11/09 16:56:49 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -43,7 +43,7 @@
 
 extern gchar *skk_jisx0208_latin_vector[];
 
-const gchar *skkconv_get_jisx0208_latin (gint ch);
-gchar *skkconv_get_jisx0208_latin_from_str (const gchar *str, gboolean store);
+const gchar *skk_conv_get_jisx0208_latin (gint ch);
+gchar *skk_conv_get_jisx0208_latin_from_str (const gchar *str, gboolean store);
 
 #endif /* HAVE_SKKCONV_JISX0208_LATIN_H */
