@@ -32,7 +32,7 @@
  *
  */
 
-/* $Id: skkconv_num.c,v 1.9 2002/03/21 21:56:00 famao Exp $ */
+/* $Id: skkconv_num.c,v 1.7 2002/11/09 16:56:49 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -75,7 +75,7 @@ static gchar *
 add_val (gchar *item, gchar *append)
 {
 	gchar *ret;
-	DEBUG_DO (printf ("skkconv_num.c : Entering add_val\n"));
+	DEBUG_DO (printf ("skk_conv_num.c : Entering add_val\n"));
 	if (!append)
 		return NULL;
 	if (!item) {
@@ -89,30 +89,17 @@ add_val (gchar *item, gchar *append)
 static gchar*
 jisx0208_latin (const gchar *number)
 {
-	const gchar *jisx0208_latin;
-	gchar *ret = NULL;
-	gint i;
-	DEBUG_DO (printf ("skkconv_num.c : Entering jisx0208_latin\n"));
+	DEBUG_DO (printf ("skk_conv_num.c : Entering jisx0208_latin\n"));
 	if (!number)
 		return NULL;
-	for ( i = 0; number[i] && isdigit (number[i]); i++) {
-		jisx0208_latin = skkconv_get_jisx0208_latin (number[i]);
-		if (ret) {
-			gchar *be_free = ret;
-			ret = g_strconcat (ret, jisx0208_latin, NULL);
-			g_free (be_free);
-		} else {
-			ret = g_strdup (jisx0208_latin);
-		}
-	}
-	return ret;
+	return  skk_conv_get_jisx0208_latin_from_str (number, TRUE);
 }
 
 static gchar *
 num_recompute (const gchar *number)
 {
 	gchar *ret = NULL;
-	DEBUG_DO (printf ("skkconv_num.c : Entering num_recompute\n"));
+	DEBUG_DO (printf ("skk_conv_num.c : Entering num_recompute\n"));
 	if (!number)
 		return NULL;
 	ret = g_strdup (number);
@@ -207,7 +194,7 @@ num_type_3 (const gchar *number)
 	gint i;
 	gint count;
 
-	DEBUG_DO (printf ("skkconv_num.c : Entering num_type_3\n"));
+	DEBUG_DO (printf ("skk_conv_num.c : Entering num_type_3\n"));
 	if (!number)
 		return NULL;
 	len = strlen (number);
@@ -278,7 +265,7 @@ num_type_2 (const gchar *number)
 	gchar *tmp_result;
 	gint   num = 0;
 	gint   i;
-	DEBUG_DO (printf ("skkconv_num.c : Entering num_type_2\n"));
+	DEBUG_DO (printf ("skk_conv_num.c : Entering num_type_2\n"));
 	if (!number)
 		return NULL;
 	for (i = 0; number[i]; i++) {
@@ -304,7 +291,7 @@ num_type_9 (const gchar *number)
 	gint len;
 	gint val;
 	const gchar *res1,*res2;
-	DEBUG_DO (printf ("skkconv_num.c : Entering num_type_9\n"));
+	DEBUG_DO (printf ("skk_conv_num.c : Entering num_type_9\n"));
 	/* FIXME better handling is needed if len != 2 */
 	if (!number)
 		return NULL;
@@ -313,9 +300,9 @@ num_type_9 (const gchar *number)
 		return strdup (number);
 	}
 	val = atoi (&number[1]);
-	res1 = skkconv_get_jisx0208_latin (number[0]);
+	res1 = skk_conv_get_jisx0208_latin (number[0]);
 	res2 = kanji_num_table[val];
-	ret = g_strconcat (res1,res2,NULL);
+	ret = g_strconcat (res1 , res2, NULL);
 	return ret;
 }
 
@@ -323,7 +310,7 @@ static gchar *
 num_identity (const gchar *number)
 {
 	gchar *ret = NULL;
-	DEBUG_DO (printf ("skkconv_num.c : Entering num_identity\n"));
+	DEBUG_DO (printf ("skk_conv_num.c : Entering num_identity\n"));
 	if (!number)
 		return NULL;
 	ret = g_strdup (number);
@@ -335,11 +322,9 @@ skk_num_jisx0208_latin (const gchar *direction_word, const gchar *number)
 {
 	gchar *sep;
 	gchar *copy;
-	gchar *trans = NULL;
-	const gchar *jisx0208_latin;
+	gchar *jisx0208_latin;
 	gchar *ret = NULL;
-	int i;
-	DEBUG_DO (printf ("skkconv_num.c : Entering skk_num_jisx0208_latin\n"));
+	DEBUG_DO (printf ("skk_conv_num.c : Entering skk_num_jisx0208_latin\n"));
 	if (!direction_word || !number)
 		return NULL;
 	copy = g_strdup (direction_word);
@@ -350,18 +335,10 @@ skk_num_jisx0208_latin (const gchar *direction_word, const gchar *number)
 	}
 	*sep = '\0';
 	sep += 2;
-	for (i = 0; number[i] && isdigit (number[i]); i++) {
-		jisx0208_latin = skkconv_get_jisx0208_latin (number[i]);
-		if (trans) {
-			gchar *be_free = trans;
-			trans = g_strconcat (trans, jisx0208_latin, NULL);
-			g_free (be_free);
-		} else {
-			trans = g_strdup (jisx0208_latin);
-		}
-	}
-	ret = g_strconcat (copy, trans, NULL);
-	g_free (trans);
+	jisx0208_latin = skk_conv_get_jisx0208_latin_from_str (number, TRUE);
+	ret = g_strconcat (copy, jisx0208_latin, sep, NULL);
+	g_free (jisx0208_latin);
+	g_free (copy);
 	return ret;
 }
 
@@ -371,7 +348,7 @@ skk_num_identity (const gchar *direction_word, const gchar *number)
 	gchar *sep;
 	gchar *copy;
 	gchar *ret = NULL;
-	DEBUG_DO (printf ("skkconv_num.c : Entering skk_num_identity\n"));
+	DEBUG_DO (printf ("skk_conv_num.c : Entering skk_num_identity\n"));
 	if (!direction_word || !number)
 		return NULL;
 	copy = g_strdup (direction_word);
@@ -393,7 +370,7 @@ skk_num_recompute (const gchar *direction_word, const gchar *number)
 	gchar *sep;
 	gchar *copy;
 	gchar *ret = NULL;
-	DEBUG_DO (printf ("skkconv_num.c : Entering skk_num_recompute\n"));
+	DEBUG_DO (printf ("skk_conv_num.c : Entering skk_num_recompute\n"));
 	if (!direction_word || !number)
 		return NULL;
 	copy = g_strdup (direction_word);
@@ -412,27 +389,28 @@ skk_num_recompute (const gchar *direction_word, const gchar *number)
 gchar *
 skk_num_translate (const gchar *direction_word, const GList *number_list, gint list_count, gint *type)
 {
-	gchar *sep;
-	gchar *sep2;
-	gchar *copy;
-	gchar *copy2;
+	gchar *sep, *sep2;
+	gchar *copy, *copy2;
+	gchar *num_translate;
 	gchar *tmp_result;
 	gchar *ret = NULL;
 	guint  found_num = 0;
-	DEBUG_DO (printf ("skkconv_num.c : Entering skk_num_translate\n"));
+	DEBUG_DO (printf ("skk_conv_num.c : Entering skk_num_translate\n"));
 	if (!direction_word || !number_list)
 		return NULL;
 	copy = copy2 = g_strdup (direction_word);
+	/* Find Separator */
 	sep = sep2 = strchr (copy, '#');
 	if (!sep) {
 		g_free (copy);
 		return NULL;
 	}
 	while (found_num < list_count ) {
-		DEBUG_DO (printf ("%s\n",sep2));
 		while (TRUE) {
+			/* Increment index */
 			sep2++;
-			if (!isdigit (*sep2)) {
+			if (!isdigit (*sep2)) { /* If *sep2 is digit , ex) #1 */
+				/* Not a Separator, Try Next */
 			    sep = sep2 = strchr (sep2, '#');
 				/* TODO */
 				if (!sep) {
@@ -440,10 +418,8 @@ skk_num_translate (const gchar *direction_word, const GList *number_list, gint l
 					return NULL;
 				}
 			} else {
-				DEBUG_DO (printf ("skk_num_translate: found separator\n"));
-				DEBUG_DO (printf ("%s\n",sep2));
 				*sep = '\0';
-				tmp_result = add_val (ret,copy2);
+				tmp_result = add_val (ret, copy2);
 				if (ret) {
 					g_free (ret);
 					ret = tmp_result;
@@ -453,57 +429,58 @@ skk_num_translate (const gchar *direction_word, const GList *number_list, gint l
 				break;
 			}
 		}
-		tmp_result = NULL;
+		num_translate = NULL;
 		if (type) {
 			*type = atoi (sep2);
 		}
 		switch (*sep2) {
 			case '0':
-				tmp_result = num_identity ( (const gchar*)g_list_nth_data ((GList*)number_list, found_num - 1));
+				num_translate = num_identity ( (const gchar*)g_list_nth_data ((GList*)number_list, found_num - 1));
 				break;
 			case '1':
-				tmp_result = jisx0208_latin ( (const gchar*) g_list_nth_data ((GList*)number_list, found_num - 1));
+				num_translate = jisx0208_latin ( (const gchar*) g_list_nth_data ((GList*)number_list, found_num - 1));
 				break;
 			case '2':
-				tmp_result = num_type_2 ( (const gchar*) g_list_nth_data ((GList*)number_list, found_num - 1));
+				num_translate = num_type_2 ( (const gchar*) g_list_nth_data ((GList*)number_list, found_num - 1));
 				break;
 			case '3':
-				tmp_result = num_type_3 ( (const gchar*) g_list_nth_data ((GList*)number_list, found_num - 1));
+				num_translate = num_type_3 ( (const gchar*) g_list_nth_data ((GList*)number_list, found_num - 1));
 				break;
 			case '4':
-				tmp_result = num_recompute ( (const gchar*) g_list_nth_data ((GList*)number_list, found_num - 1));
+				num_translate = num_recompute ( (const gchar*) g_list_nth_data ((GList*)number_list, found_num - 1));
 				break;
 			case '5':
-				tmp_result = num_type_5 ( (const gchar*) g_list_nth_data ((GList*)number_list, found_num - 1));
+				num_translate = num_type_5 ( (const gchar*) g_list_nth_data ((GList*)number_list, found_num - 1));
 				break;
 			case '9':
-				tmp_result = num_type_9 ( (const gchar*) g_list_nth_data ((GList*)number_list, found_num - 1));
+				num_translate = num_type_9 ( (const gchar*) g_list_nth_data ((GList*)number_list, found_num - 1));
 				break;
 			default:
 				/* TODO */
-				tmp_result = num_identity ((const gchar*)g_list_nth_data ((GList*)number_list, found_num - 1));
+				num_translate = num_identity ((const gchar*)g_list_nth_data ((GList*)number_list, found_num - 1));
 				break;
 		}
-		if (!tmp_result) {
+		if (!num_translate) {
 			if (ret) {
 				g_free (ret);
 			}
 			g_free (copy);
 			return NULL;
 		}
-		DEBUG_DO (printf ("skk_num_translate: tmp_result %s\n",tmp_result));
 		if (ret) {
-			gchar *be_free = ret;
-			ret = g_strconcat (ret,tmp_result,NULL);
-			g_free (be_free);
+			tmp_result = ret;
+			ret = g_strconcat (ret, num_translate , NULL);
+			g_free (tmp_result);
 		} else {
-			ret = g_strdup (tmp_result);
+			ret = g_strdup (num_translate);
 		}
+		g_free (num_translate);
 	}
+
 	if (copy2) {
-		gchar *be_free = ret;
+		tmp_result = ret;
 		ret = g_strconcat (ret, copy2, NULL);
-		g_free (be_free);
+		g_free (tmp_result);
 	}
 	return ret;
 }
@@ -514,6 +491,7 @@ main (void)
 {
 	GList *test = NULL;
 	gchar *string = "Ϥ#0#0ؤ#1#4ۤ#9#2#1#3#5#3|#3"; 
+#if 0
 	test = g_list_append (test,"1234");
 	test = g_list_append (test,"5678");
 	test = g_list_append (test,"3347");
@@ -525,13 +503,20 @@ main (void)
 	test = g_list_append (test,"111111110000");
 	test = g_list_append (test,"3004426");
 	test = g_list_append (test,"111111110000");
-	g_message ("%s\n",skk_num_translate (string,test,11,NULL));
+	string = skk_num_translate (string, test, 11, NULL);
+	g_message ("%s\n",string);
+	g_free (string);
+	g_list_free (test);
+#endif
 	string = skk_num_identity ("#0#0#1#4","1234");
 	g_message ("%s\n",string);
+	g_free (string);
 	string = skk_num_identity (string,"5678");
 	g_message ("%s\n",string);
+	g_free (string);
 	string = skk_num_jisx0208_latin (string,"3347");
 	g_message ("%s\n",string);
+	g_free (string);
 	return 0;
 }
 #endif
