@@ -32,7 +32,7 @@
  *
  */
 
-/* $Id: skkel.h,v 1.1.1.1 2002/04/06 02:23:29 famao Exp $ */
+/* $Id: skkel.h,v 1.4 2002/11/09 16:56:49 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -40,8 +40,16 @@
 #ifndef HAVE_SKKEL_H
 #define HAVE_SKKEL_H
 #include <glib.h>
+#include <skktypes.h>
 
 typedef struct _SkkLisp SkkLisp;
+typedef struct _SkkSymbol SkkSymbol;
+
+struct _SkkSymbol
+{
+	SkkLispSymbolType symbol;
+	gchar *name;
+};
 
 struct _SkkLisp
 {
