@@ -32,7 +32,7 @@
  *
  */
 
-/* $Id: skkfunc.c,v 1.3 2002/04/11 02:19:04 famao Exp $ */
+/* $Id: skkfunc.c,v 1.8 2002/11/09 16:56:49 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -42,7 +42,7 @@
 #include "skkfunc.h"
 #include "skkkeysym.h"
 #include "skkfunc_private.h"
-#include "skkbuffer.h"
+#include "skkutils.h"
 
 SkkFunc *
 skk_func_new (void)
@@ -54,19 +54,46 @@ skk_func_new (void)
 	for (i = 0; i < DefaultItemSize; i++) {
 		skk_func_add_item (ret, &(DefaultItem[i]));
 	}
+	ret->ref_count++;
 	return ret;
 }
 
+static void
+item_free_func (gpointer data, gpointer user_data)
+{
+	if (data) {
+		skk_func_item_destroy ((SkkFuncItem *)data);
+		data = NULL;
+	}
+}
+
 void
 skk_func_destroy (SkkFunc *func)
 {
 	if (!func)
 		return;
+	func->ref_count--;
 	if (func->ref_count > 0) {
-		skk_func_unref (func);
 		return;
 	}
-	/* TODO */
+	if (func->jmode_func_list) {
+		skk_utils_list_free (func->jmode_func_list, TRUE, item_free_func, NULL);
+		func->jmode_func_list = NULL;
+	}
+	if (func->abbrev_func_list) {
+		skk_utils_list_free (func->abbrev_func_list, TRUE, item_free_func, NULL);
+		func->abbrev_func_list = NULL;
+	}
+	if (func->latin_func_list) {
+		skk_utils_list_free (func->latin_func_list, TRUE, item_free_func, NULL);
+		func->latin_func_list = NULL;
+	}
+	if (func->jisx0208_latin_func_list) {
+		skk_utils_list_free (func->jisx0208_latin_func_list, TRUE, item_free_func, NULL);
+		func->jisx0208_latin_func_list = NULL;
+	}
+	g_free (func);
+	return;
 }
 
 void
@@ -253,7 +280,7 @@ skk_func_item_new (guint mode, guint submode, guint key, guint mask, guint actua
 }
 
 void
-skk_func_item_free (SkkFuncItem *item)
+skk_func_item_destroy (SkkFuncItem *item)
 {
 	if (!item)
 		return;
