@@ -32,7 +32,7 @@
  *
  */
 
-/* $Id: skkfunc.h,v 1.2 2002/04/11 02:19:04 famao Exp $ */
+/* $Id: skkfunc.h,v 1.6 2002/11/09 16:56:49 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -41,7 +41,7 @@
 #define HAVE_SKKFUNC_H
 
 #include <glib.h>
-#include "skktypes.h"
+#include <skktypes.h>
 
 typedef struct _SkkFunc SkkFunc;
 typedef struct _SkkFuncItem SkkFuncItem;
@@ -55,7 +55,7 @@ struct _SkkFuncResult {
 struct _SkkFuncItem {
 	guint mode;
 	guint submode;
-	guint key;
+	guint key ;
 	guint mask;
 	guint result;
 	guint actual_key;
@@ -80,7 +80,7 @@ SkkFuncResult* skk_func_do_query (SkkFunc *func, guint mode, guint submode, guin
 
 /* SkkFuncItem */
 void skk_func_add_item (SkkFunc *func, SkkFuncItem *item);
-void skk_func_item_free (SkkFuncItem *item);
+void skk_func_item_destroy (SkkFuncItem *item);
 SkkFuncItem *skk_func_item_new (guint mode, guint submode, guint key, guint mask, guint actual_key, guint result);
 
 
