@@ -32,7 +32,7 @@
  *
  */
 
-/* $Id: skkfunc_private.h,v 1.7 2002/04/11 02:19:04 famao Exp $ */
+/* $Id: skkfunc_private.h,v 1.10 2002/11/09 16:56:49 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -49,8 +49,8 @@ static SkkFuncItem DefaultItem[] =
 	{ (SKK_FUNC_JMODE | SKK_FUNC_ABBREV) , QUERY_ALL, SKK_VK_H, SKK_CONTROL_MASK, SKKFUNC_BACKWARD_DELETE, 0, FALSE},
 	{ (SKK_FUNC_JMODE | SKK_FUNC_ABBREV), QUERY_ALL, SKK_VK_h, SKK_CONTROL_MASK, SKKFUNC_BACKWARD_DELETE, 0, FALSE},
 	/* Cancel */
-	{ (SKK_FUNC_JMODE | SKK_FUNC_ABBREV), QUERY_ALL, SKK_VK_g, SKK_CONTROL_MASK, SKKFUNC_CANCEL, 0, FALSE},
-	{ (SKK_FUNC_JMODE | SKK_FUNC_ABBREV), QUERY_ALL, SKK_VK_G, SKK_CONTROL_MASK, SKKFUNC_CANCEL, 0, FALSE},
+	{ (SKK_FUNC_JMODE | SKK_FUNC_ABBREV), ~QUERY_NONE, SKK_VK_g, SKK_CONTROL_MASK, SKKFUNC_CANCEL, 0, FALSE},
+	{ (SKK_FUNC_JMODE | SKK_FUNC_ABBREV), ~QUERY_NONE, SKK_VK_G, SKK_CONTROL_MASK, SKKFUNC_CANCEL, 0, FALSE},
 	/* Category Change */
 	{ SKK_FUNC_JMODE, QUERY_NORMAL, SKK_VK_q, SKK_NONE_MASK, SKKFUNC_CATEGORY_CHANGE, 0, FALSE},
 	{ SKK_FUNC_ABBREV, QUERY_NORMAL, SKK_VK_q, SKK_CONTROL_MASK, SKKFUNC_CATEGORY_CHANGE, 0, FALSE},
@@ -61,8 +61,15 @@ static SkkFuncItem DefaultItem[] =
 	{ (SKK_FUNC_JMODE | SKK_FUNC_ABBREV), QUERY_NORMAL, SKK_VK_SPACE, SKK_ALT_MASK, SKKFUNC_COMPLETE_TRANSLATE, 0, FALSE},
 	/* CodeTable */
 	{ SKK_FUNC_JMODE, QUERY_ALL, SKK_VK_BACK_SLASH, SKK_ALL_MASK, SKKFUNC_CODETABLE, 0, FALSE},
+	/* Cursor */
+	{ (SKK_FUNC_JMODE | SKK_FUNC_ABBREV), QUERY_NORMAL, SKK_VK_b, SKK_CONTROL_MASK, SKKFUNC_CURSOR_LEFT, 0, FALSE},
+	{ (SKK_FUNC_JMODE | SKK_FUNC_ABBREV), QUERY_NORMAL, SKK_VK_B, SKK_CONTROL_MASK, SKKFUNC_CURSOR_LEFT, 0, FALSE},
+	{ (SKK_FUNC_JMODE | SKK_FUNC_ABBREV), QUERY_NORMAL, SKK_VK_f, SKK_CONTROL_MASK, SKKFUNC_CURSOR_RIGHT, 0, FALSE},
+	{ (SKK_FUNC_JMODE | SKK_FUNC_ABBREV), QUERY_NORMAL, SKK_VK_F, SKK_CONTROL_MASK, SKKFUNC_CURSOR_RIGHT, 0, FALSE},
 	/* Kakutei */
 	{ (SKK_FUNC_JMODE | SKK_FUNC_ABBREV), QUERY_ALL, SKK_VK_ENTER, SKK_ALL_MASK, SKKFUNC_KAKUTEI, SKK_VK_ENTER, FALSE},
+	{ (SKK_FUNC_JMODE | SKK_FUNC_ABBREV), QUERY_ALL, SKK_VK_m, SKK_CONTROL_MASK, SKKFUNC_KAKUTEI, SKK_VK_ENTER, FALSE},
+	{ (SKK_FUNC_JMODE | SKK_FUNC_ABBREV), QUERY_ALL, SKK_VK_M, SKK_CONTROL_MASK, SKKFUNC_KAKUTEI, SKK_VK_ENTER, FALSE},
 	{ (SKK_FUNC_JMODE | SKK_FUNC_ABBREV), QUERY_ALL, SKK_VK_j, SKK_CONTROL_MASK, SKKFUNC_KAKUTEI, 0, FALSE},
 	{ (SKK_FUNC_JMODE | SKK_FUNC_ABBREV), QUERY_ALL, SKK_VK_J, SKK_CONTROL_MASK, SKKFUNC_KAKUTEI, 0, FALSE},
 	/* Kana */
