@@ -32,7 +32,7 @@
  *
  */
 
-/* $Id: $ */
+/* $Id: skkgadget.h,v 1.3 2002/11/09 16:56:49 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -40,7 +40,7 @@
 #ifndef HAVE_SKKGADGET_H
 #define HAVE_SKKGADGET_H
 #include <glib.h>
-#include "skkconv_jisx0208_latin.h"
+#include <skkconv_jisx0208_latin.h>
 
 typedef struct _SkkJDateNode SkkJDateNode;
 
