@@ -32,7 +32,7 @@
  *
  */
 
-/* $Id: skkkeymap.c,v 1.12.2.1 2002/04/12 21:54:11 famao Exp $ */
+/* $Id: skkkeymap.c,v 1.21 2002/11/09 16:56:49 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -41,6 +41,7 @@
 #include "skkkeysym.h"
 #include "skktypes.h"
 #include "skkfunc.h"
+#include "skkconv_kana.h"
 
 static gint
 keymap_jmode (SkkBuffer *buf, SkkFuncResult *result)
@@ -56,7 +57,7 @@ keymap_jmode (SkkBuffer *buf, SkkFuncResult *result)
 			g_message ("in BACKWARD_DELETE");
 #endif
 			if (jstatus == SKK_CHOICE &&
-					!skk_conf_get_delete_implies_kakutei (skkbuf_conf (buf))) {
+					!skk_conf_get_bool_by_name (skkbuf_conf (buf), "skk-delete-implies-kakutei")) {
 				skk_buffer_undo (buf);
 				return SKK_NOTHING;
 			}
@@ -88,6 +89,7 @@ keymap_jmode (SkkBuffer *buf, SkkFuncResult *result)
 			switch (jstatus) {
 				case SKK_NONE:
 					skk_buffer_clear (buf);
+					return SKK_NOTRANS;
 					break;
 				case SKK_HENKAN:
 				case SKK_OKURI:
@@ -99,13 +101,21 @@ keymap_jmode (SkkBuffer *buf, SkkFuncResult *result)
 					skk_buffer_clear (buf);
 					break;
 			}
+#if 0
 			if (result->actual_key == SKK_VK_ENTER) {
-				if (skk_conf_get_egg_like_newline (skkbuf_conf (buf))) {
+				if (skk_conf_get_egg_like_newline (skkbuf_conf (buf)) && jstatus != SKK_NONE) {
 					return SKK_NOTHING;
 				} else {
 					return SKK_NOTRANS;
 				}
 			}
+#else
+			if (skk_conf_get_bool_by_name (skkbuf_conf (buf), "skk-egg-like-newline")) {
+				return SKK_NOTHING;
+			} else {
+				return SKK_NOTRANS;
+			}
+#endif
 			return SKK_NOTHING;
 			break;
 		case SKKFUNC_COMPLETE_TRANSLATE:
@@ -132,8 +142,12 @@ keymap_jmode (SkkBuffer *buf, SkkFuncResult *result)
 				return SKK_NOTHING;
 			} else if (jstatus == SKK_CHOICE) {
 				if (skk_buffer_has_next_candidate (buf)) {
+					static gint candidate_count = -1;
+					if (candidate_count == -1) {
+						candidate_count = skk_conf_get_num_by_name (skkbuf_conf (buf), "skk-candidate-count");
+					}
 					/* TODO */
-					if (buf->candidate_count > 3) {
+					if (buf->candidate_count >= candidate_count) {
 						buf->candidate_count++;
 						skk_buffer_lookup_emit (buf);
 					} else {
@@ -188,7 +202,7 @@ keymap_jmode (SkkBuffer *buf, SkkFuncResult *result)
 #endif
 			if (jstatus == SKK_HENKAN) {
 				if (skkbuf_is_completion (buf)) {
-					if (skk_conf_get_dabbrev_like_completion (skkbuf_conf (buf))) {
+					if (skk_conf_get_bool_by_name (skkbuf_conf (buf), "skk-dabbrev-like-completion")) {
 						skk_buffer_get_completion (buf);
 					} else {
 						skk_buffer_set_next_completion (buf);
@@ -203,6 +217,11 @@ keymap_jmode (SkkBuffer *buf, SkkFuncResult *result)
 #ifdef SKKKEYMAP_DEBUG
 			g_message ("in CANCEL");
 #endif
+			if (jstatus == SKK_CHOICE) {
+				buf->candidate_count = 0;
+				skk_buffer_undo (buf);
+				return SKK_NOTHING;
+			}
 			skk_buffer_clear (buf);
 			return SKK_NOTHING;
 		case SKKFUNC_MODE_CHANGE:
@@ -295,6 +314,15 @@ keymap_jmode (SkkBuffer *buf, SkkFuncResult *result)
 			}
 			return SKK_NOTHING;
 			break;
+		case SKKFUNC_CURSOR_LEFT:
+			skk_buffer_move_left (buf);
+			g_message ("fuge");
+			return SKK_NOTHING;
+			break;
+		case SKKFUNC_CURSOR_RIGHT:
+			skk_buffer_move_right (buf);
+			return SKK_NOTHING;
+			break;
 		default:
 			break;
 	}
@@ -321,6 +349,7 @@ keymap_abbrev (SkkBuffer *buf, SkkFuncResult *result)
 			}
 			skk_buffer_commit (buf, TRUE);
 			skk_buffer_clear (buf);
+#if 0
 			if (result->actual_key == SKK_VK_ENTER) {
 				if (skk_conf_get_egg_like_newline (skkbuf_conf (buf))) {
 					return SKK_NOTHING;
@@ -328,6 +357,13 @@ keymap_abbrev (SkkBuffer *buf, SkkFuncResult *result)
 					return SKK_NOTRANS;
 				}
 			}
+#else
+			if (skk_conf_get_bool_by_name (skkbuf_conf (buf), "skk-egg-like-newline")) {
+				return SKK_NOTHING;
+			} else {
+				return SKK_NOTRANS;
+			}
+#endif
 			return SKK_NOTHING;
 			break;
 		case SKKFUNC_CATEGORY_CHANGE:
@@ -340,7 +376,7 @@ keymap_abbrev (SkkBuffer *buf, SkkFuncResult *result)
 			if (qstatus == QUERY_DONE) {
 				if (skkbuf_okuri (buf)) {
 					skkbuf_candidate_num (buf) = 0;
-					if (skk_conf_get_delete_okuri_when_quit (skkbuf_conf (buf))) {
+					if (skk_conf_get_bool_by_name (skkbuf_conf (buf), "skk-delete-okuri-when-quit")) {
 						/* TODO dirty hack */
 						g_free (skkbuf_okuri (buf));
 						g_free (skkbuf_okurip (buf));
@@ -360,7 +396,7 @@ keymap_abbrev (SkkBuffer *buf, SkkFuncResult *result)
 			break;
 		case SKKFUNC_COMPLETE_NEXT:
 			if (skkbuf_is_completion (buf)) {
-				if (skk_conf_get_dabbrev_like_completion (skkbuf_conf (buf))) {
+				if (skk_conf_get_bool_by_name (skkbuf_conf (buf), "skk-dabbrev-like-completion")) {
 					skk_buffer_get_completion (buf);
 				} else {
 					skk_buffer_set_next_completion (buf);
@@ -393,8 +429,12 @@ keymap_abbrev (SkkBuffer *buf, SkkFuncResult *result)
 				skk_query_set_first (skkbuf_query (buf));
 				skk_buffer_query_dict (buf);
 			} else if (qstatus == QUERY_DONE) {
+				static gint candidate_count = -1;
+				if (candidate_count == -1) {
+					candidate_count = skk_conf_get_num_by_name (skkbuf_conf (buf), "skk-candidate-count");
+				}
 				if (skk_buffer_has_next_candidate (buf)) {
-					if (buf->candidate_count > 3) {
+					if (buf->candidate_count >= candidate_count) {
 						buf->candidate_count++;
 						skk_buffer_lookup_emit (buf);
 					} else {
@@ -410,6 +450,14 @@ keymap_abbrev (SkkBuffer *buf, SkkFuncResult *result)
 			skk_buffer_undo (buf);
 			return SKK_NOTHING;
 			break;
+		case SKKFUNC_CURSOR_LEFT:
+			skk_buffer_move_left (buf);
+			return SKK_NOTHING;
+			break;
+		case SKKFUNC_CURSOR_RIGHT:
+			skk_buffer_move_right (buf);
+			return SKK_NOTHING;
+			break;
 		default:
 			break;
 	}
@@ -452,7 +500,7 @@ keymap_jisx0208_latin (SkkBuffer *buf, SkkFuncResult *result)
 }
 
 gint
-skk_keymap_do_func (SkkBuffer *buf, SkkFunc *func, gint ch, gint mask)
+skk_keymap_do_func (SkkBuffer *buf, SkkFunc *func, guint ch, guint mask)
 {
 	SkkFuncResult *result;
 	SkkStatus status;
@@ -480,7 +528,19 @@ skk_keymap_do_func (SkkBuffer *buf, SkkFunc *func, gint ch, gint mask)
 	} else {
 		switch (status) {
 			case SKK_J_MODE:
-				ret = keymap_jmode (buf, result);
+				{
+					gchar *check;
+					if (isalnum (ch) &&
+							(mask & (SKK_NONE_MASK | SKK_SHIFT_MASK)) && skkbuf_b (buf)) {
+						check = g_strdup_printf ("%s%c", skkbuf_b (buf), ch);
+						if (skk_conv_is_exist (buf->rule, check)) {
+							ret = SKK_INSERT_CHAR;
+							g_free (check);
+							break;
+						}
+					}
+					ret = keymap_jmode (buf, result);
+				}
 				break;
 			case SKK_ABBREV_MODE:
 				ret = keymap_abbrev (buf, result);
@@ -530,16 +590,41 @@ main (void)
 {
 	SkkBuffer *buf;
 	SkkFunc *func;
+	SkkQuery *query;
+	SkkQueryItem *item;
+	SkkConvRule *rule;
 	gchar tmp[8192];
 	int i;
 	buf = skk_buffer_new ();
 	func = skk_func_new ();
+	query = skk_query_new_with_path ("./.libs");
+	rule = skk_conv_rule_new ();
+#if 1
+	item = skk_query_item_new (query, SKKQUERY_LOCAL);
+	skk_query_add_item (query, item);
+	item = skk_query_item_new (query, SKKQUERY_SERVER);
+	skk_query_add_item (query, item);
+#else
+	skk_query_add_item (query, skk_query_item_new (query, SKKQUERY_LOCAL));
+	skk_query_add_item (query, skk_query_item_new (query, SKKQUERY_SERVER));
+#endif
+	skk_query_add_item (query, skk_query_item_new (query, SKKQUERY_LOOK));
 	skk_buffer_add_preedit_listener (buf, listener, NULL);
 	skk_buffer_add_result_listener (buf, listener, NULL);
+	printf ("%p\n", query);
+	skk_buffer_set_query (buf, query);
+#if 1
+	skk_buffer_set_rule (buf, rule);
+#endif
 	skk_keymap_do_func (buf, func, SKK_VK_q, SKK_CONTROL_MASK);
 	while (fgets (tmp, 8192, stdin)) {
 		for (i = 0;  tmp[i]; i++) {
+#if 0
 			skk_keymap_do_func (buf, func, tmp[i], SKK_ALL_MASK);
+#else
+			skk_keymap_do_func (buf, func, tmp[i], SKK_NONE_MASK);
+			printf ("%p\n", buf->query);
+#endif
 		}
 	}
 	return 0;
