@@ -32,17 +32,17 @@
  *
  */
 
-/* $Id: $ */
+/* $Id: skkkeymap.h,v 1.2 2002/11/09 16:56:49 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
 #ifndef HAVE_SKKKEYMAP_H
 #define HAVA_SKKKEYMAP_H
 #include <glib.h>
-#include "skkbuffer.h"
-#include "skkfunc.h"
-#include "skkgadget.h"
+#include <skkbuffer.h>
+#include <skkfunc.h>
+#include <skkgadget.h>
 
-gint skk_keymap_do_func (SkkBuffer *buf, SkkFunc *func, gint key, gint mask);
+gint skk_keymap_do_func (SkkBuffer *buf, SkkFunc *func, guint key, guint mask);
 
 #endif /* HAVE_SKKKEYMAP_H */
