@@ -32,20 +32,19 @@
  *
  */
 
-/* $Id: skkldictimpl.c,v 1.1.2.4 2002/04/21 01:27:44 famao Exp $ */
+/* $Id: skkldictimpl.c,v 1.5 2002/11/09 16:56:49 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
 
 #include <string.h>
-#include <db.h>
 #include <fcntl.h>
 #include <glib.h>
 #include "skkldictimpl.h"
 #include "skkdictitem.h"
 #include "skktypes.h"
 
-static SkkQueryFunctionImpl *get_new (void);
+static SkkQueryFunctionImpl *get_new (SkkQuery *query);
 static GList *do_query (SkkQueryFunctionImpl *impl, const gchar *buf, const gchar *okuri, gint *found_num);
 static void destroy (SkkQueryFunctionImpl *impl);
 static const gchar* get_info (void);
@@ -279,11 +278,13 @@ completion (SkkQueryFunctionImpl *impl, const gchar *buf)
 }
 
 static SkkQueryFunctionImpl *
-get_new (void)
+get_new (SkkQuery *query)
 {
 	SkkLdictImpl *impl;
 	impl = g_new0 (SkkLdictImpl, 1);
 	impl->impl = ldictimpl;
+	impl->file_ldict = skk_query_get_conf_string_by_name (query, "skk-jisyo");
+	impl->file_ldict_okuri = skk_query_get_conf_string_by_name (query, "skk-okuri-jisyo");
 	impl->ldict = NULL;
 	impl->ldict_okuri = NULL;
 	impl->initialized = FALSE;
@@ -306,7 +307,23 @@ static void
 destroy (SkkQueryFunctionImpl *impl)
 {
 	SkkLdictImpl *l = (SkkLdictImpl *)impl;
+	u_int32_t countp;
 	/* TODO */
+	if (l->initialized) {
+		if (!l->file_ldict) {
+			l->ldict->truncate (l->ldict, NULL, &countp, 0);
+		}
+		l->ldict->close (l->ldict, 0);
+		if (!l->file_ldict_okuri) {
+			l->ldict_okuri->truncate (l->ldict_okuri, NULL, &countp, 0);
+		}
+		l->ldict_okuri->close (l->ldict_okuri, 0);
+		l->initialized = FALSE;
+	}
+	if (l->file_ldict)
+		g_free (l->file_ldict);
+	if (l->file_ldict_okuri)
+		g_free (l->file_ldict_okuri);
 	g_free (l);
 	return;
 }
@@ -334,13 +351,29 @@ ldict_init (SkkQueryFunctionImpl *impl)
 	}
 	l->ldict->set_errfile (l->ldict, stderr);
 	l->ldict->set_errpfx (l->ldict, "iiimf-skk");
-	l->ldict->open (l->ldict, NULL, 0, DB_HASH, DB_CREATE, 0644);
+	if ((err = l->ldict->open (l->ldict, l->file_ldict, 0, DB_HASH, DB_CREATE, 0644))) {
+		printf ("ldict_init %s\n", db_strerror (err));
+		if (l->file_ldict) {
+			g_free (l->file_ldict);
+			l->file_ldict = NULL;
+			l->ldict->open (l->ldict, NULL, 0, DB_HASH, DB_CREATE, 0644);
+		}
+	}
 
 	if ((err = db_create (&(l->ldict_okuri), NULL, 0) != 0)) {
 	}
 	l->ldict_okuri->set_errfile (l->ldict_okuri, stderr);
 	l->ldict_okuri->set_errpfx (l->ldict_okuri, "iiimf-skk");
-	l->ldict_okuri->open (l->ldict_okuri, NULL, 0, DB_HASH, DB_CREATE, 0644);
+	if ((err = l->ldict_okuri->open (l->ldict_okuri, l->file_ldict_okuri, 0, DB_HASH,
+					DB_CREATE, 0644))) {
+		printf ("ldict_init %s\n", db_strerror (err));
+		if (l->file_ldict_okuri) {
+			g_free (l->file_ldict_okuri);
+			l->file_ldict_okuri = NULL;
+			l->ldict_okuri->open (l->ldict_okuri, NULL, 0, DB_HASH, DB_CREATE, 0644);
+		}
+	}
+
 	l->initialized = TRUE;
 	return TRUE;
 }
@@ -360,15 +393,15 @@ add (SkkQueryFunctionImpl *impl, const gchar *key, const gchar *okuri, const gch
 }
 
 SkkQueryFunctionImpl *
-skk_ldict_new (void)
+skk_ldict_new (SkkQuery *query)
 {
-	return get_new ();
+	return get_new (NULL);
 }
 
 SkkQueryFunctionImpl *
-impl_init (void)
+impl_init (SkkQuery *query)
 {
-	return get_new ();
+	return get_new (query);
 }
 
 #ifdef SKKLDICTIMPL_DEBUG
