@@ -32,7 +32,7 @@
  *
  */
 
-/* $Id: skkmode.c,v 1.3 2002/04/11 02:04:14 famao Exp $ */
+/* $Id: skkmode.c,v 1.5 2002/11/09 16:56:49 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -43,18 +43,36 @@
 #include "skkutils.h"
 #include "skkdebug.h"
 
-static gchar* get_status_string (SkkMode *mode);
 static void clear (SkkMode *mode);
+
+/* Get Status String */
+static gchar* get_status_string (SkkMode *mode);
+
+/* Set Status */
 static void set_status (SkkMode *mode, SkkStatus status);
-static SkkStatus get_status (SkkMode *mode);
 static void set_j_status (SkkMode *mode, SkkJStatus jstatus);
-static SkkJStatus get_j_status (SkkMode *mode);
 static void set_query_status (SkkMode *mode, SkkQueryStatus status);
 
-typedef struct _SkkModeFunc {
+/* Get Status */
+static SkkStatus get_status (SkkMode *mode);
+static SkkJStatus get_j_status (SkkMode *mode);
+static SkkQueryStatus get_query_status (SkkMode *mode);
+
+/* Prepare prefix, postfix */
+static void prepare_mark (SkkMode *mode);
+
+/* Event */
+static void status_emit (SkkMode *mode);
+static void j_status_emit (SkkMode *mode);
+static void query_status_emit (SkkMode *mode);
+
+
+typedef struct _SkkModeFunc SkkModeFunc;
+
+struct _SkkModeFunc {
 	SkkModeListener listener;
 	gpointer user_data;
-}SkkModeFunc;
+};
 
 static void
 status_emit (SkkMode *mode)
@@ -103,6 +121,58 @@ query_status_emit (SkkMode *mode)
 	return;
 }
 
+void
+prepare_mark (SkkMode *mode)
+{
+	DEBUG_DO (printf ("skkmode.c : Entering prepare_mark\n"));
+	if (!mode) return;
+	switch (mode->query_status) {
+		case QUERY_NONE:
+			if (mode->query_prefix) {
+				g_free (mode->query_prefix);
+				mode->query_prefix = NULL;
+			}
+			if (mode->query_suffix) {
+				g_free (mode->query_suffix);
+				mode->query_suffix = NULL;
+			}
+			break;
+		case QUERY_NORMAL:
+			if (mode->query_prefix) {
+				g_free (mode->query_prefix);
+			}
+			mode->query_prefix = g_strdup ("");
+			if (mode->query_suffix) {
+				g_free (mode->query_suffix);
+				mode->query_suffix = NULL;
+			}
+			break;
+		case QUERY_OKURI:
+			if (mode->query_prefix) {
+				g_free (mode->query_prefix);
+			}
+			mode->query_prefix = g_strdup ("");
+			if (mode->query_suffix) {
+				g_free (mode->query_suffix);
+			}
+			mode->query_suffix = g_strdup ("*");
+			break;
+		case QUERY_DONE:
+			if (mode->query_prefix) {
+				g_free (mode->query_prefix);
+			}
+			mode->query_prefix = g_strdup ("");
+			if (mode->query_suffix) {
+				g_free (mode->query_suffix);
+				mode->query_suffix = NULL;
+			}
+			break;
+		default:
+			break;
+	}
+	return;
+}
+
 static gchar *
 get_status_string (SkkMode *mode)
 {
@@ -113,22 +183,23 @@ get_status_string (SkkMode *mode)
 	{
 		case SKK_J_MODE:
 			if (mode->jisx0201_kana) {
-				mode->status_buf = g_strdup("Ⱦ");
+				mode->status_buf = g_strdup (mode->jisx0201_kana_str);
 				break;
 			}
 			if (mode->katakana)
-				mode->status_buf = g_strdup ("");
+				mode->status_buf = g_strdup (mode->katakana_str);
 			else
-				mode->status_buf = g_strdup ("");
+				mode->status_buf = g_strdup (mode->hirakana_str);
 			break;
 		case SKK_LATIN_MODE:
-			mode->status_buf = g_strdup ("SKK");
+			mode->status_buf = g_strdup (mode->latin_str);
 			break;
 		case SKK_JISX0208_LATIN_MODE:
-			mode->status_buf = g_strdup ("");
+			mode->status_buf = g_strdup (mode->jisx0208_latin_str);
 			break;
 		case SKK_ABBREV_MODE:
-			mode->status_buf = g_strdup ("a");
+			/* FIXME need to be customizable ? */
+			mode->status_buf = g_strdup (mode->abbrev_str);
 			break;
 		default:
 			mode->status_buf = g_strdup ("Unknown");
@@ -214,6 +285,15 @@ set_j_status (SkkMode *mode, SkkJStatus jstatus)
 	return;
 }
 
+static SkkQueryStatus
+get_query_status (SkkMode *mode)
+{
+	DEBUG_DO (printf ("skkmode.c : Entering get_query_status\n"));
+	if (!mode)
+		return QUERY_NONE;
+	return mode->query_status;
+}
+
 static SkkJStatus
 get_j_status (SkkMode *mode)
 {
@@ -240,14 +320,21 @@ skk_mode_new (void)
 	ret->status = SKK_J_MODE;
 	ret->query_status = QUERY_NONE;
 	ret->jstatus = SKK_NONE;
+	ret->latin_str = g_strdup ("SKK");
+	ret->hirakana_str = g_strdup ("");
+	ret->katakana_str = g_strdup ("");
+	ret->jisx0208_latin_str = g_strdup ("");
+	ret->jisx0201_kana_str = g_strdup ("Ⱦ");
+	ret->abbrev_str = g_strdup ("a");
 	ret->clear = clear;
 	ret->set_status = set_status;
 	ret->set_j_status = set_j_status;
 	ret->set_query_status = set_query_status;
 	ret->get_status = get_status;
 	ret->get_status_string = get_status_string;
-	ret->get_query_status = skk_mode_get_query_status;
+	ret->get_query_status = get_query_status;
 	ret->get_j_status = get_j_status;
+	ret->prepare_mark = prepare_mark;
 	return ret;
 }
 
@@ -268,7 +355,19 @@ skk_mode_destroy (SkkMode *mode)
 	if (mode->query_status_listener) {
 		skk_utils_list_free (mode->query_status_listener, TRUE, NULL, NULL);
 		mode->query_status_listener = NULL;
-	}	
+	}
+	if (mode->latin_str)
+		g_free (mode->latin_str);
+	if (mode->hirakana_str)
+		g_free (mode->hirakana_str);
+	if (mode->katakana_str)
+		g_free (mode->katakana_str);
+	if (mode->jisx0208_latin_str)
+		g_free (mode->jisx0208_latin_str);
+	if (mode->jisx0201_kana_str)
+		g_free (mode->jisx0201_kana_str);
+	if (mode->abbrev_str)
+		g_free (mode->abbrev_str);
 	g_free (mode);
 	return;
 }
@@ -323,57 +422,6 @@ skk_mode_get_katakana (SkkMode *mode)
 	return mode->katakana;
 }
 
-void
-skk_mode_prepare_mark (SkkMode *mode)
-{
-	DEBUG_DO (printf ("skkmode.c : Entering skk_mode_prepare_mark\n"));
-	if (!mode) return;
-	switch (mode->query_status) {
-		case QUERY_NONE:
-			if (mode->query_prefix) {
-				g_free (mode->query_prefix);
-				mode->query_prefix = NULL;
-			}
-			if (mode->query_suffix) {
-				g_free (mode->query_suffix);
-				mode->query_suffix = NULL;
-			}
-			break;
-		case QUERY_NORMAL:
-			if (mode->query_prefix) {
-				g_free (mode->query_prefix);
-			}
-			mode->query_prefix = g_strdup ("");
-			if (mode->query_suffix) {
-				g_free (mode->query_suffix);
-				mode->query_suffix = NULL;
-			}
-			break;
-		case QUERY_OKURI:
-			if (mode->query_prefix) {
-				g_free (mode->query_prefix);
-			}
-			mode->query_prefix = g_strdup ("");
-			if (mode->query_suffix) {
-				g_free (mode->query_suffix);
-			}
-			mode->query_suffix = g_strdup ("*");
-			break;
-		case QUERY_DONE:
-			if (mode->query_prefix) {
-				g_free (mode->query_prefix);
-			}
-			mode->query_prefix = g_strdup ("");
-			if (mode->query_suffix) {
-				g_free (mode->query_suffix);
-				mode->query_suffix = NULL;
-			}
-			break;
-		default:
-			break;
-	}
-	return;
-}
 
 void
 skk_mode_set_query_status (SkkMode *mode, SkkQueryStatus status)
@@ -404,7 +452,11 @@ skk_mode_get_query_status (SkkMode *mode)
 {
 	DEBUG_DO (printf ("skkmode.c : Entering skk_mode_get_query_status\n"));
 	if (!mode) return QUERY_NONE;
-	return mode->query_status;
+	if (mode->get_query_status) {
+		return mode->get_query_status (mode);
+	} else {
+		return QUERY_NONE;
+	}
 }
 
 const gchar*
@@ -482,12 +534,25 @@ skk_mode_get_status_string (SkkMode *mode)
 	}
 }
 
+void
+skk_mode_prepare_mark (SkkMode *mode)
+{
+	if (!mode)
+		return;
+	if (mode->prepare_mark) {
+		mode->prepare_mark (mode);
+	}
+	return;
+}
+
 void
 skk_mode_add_status_listener (SkkMode *mode, SkkModeListener listener, gpointer user_data)
 {
 	SkkModeFunc *func;
 	if (!mode)
 		return;
+	if (!listener)
+		return;
 	func = g_new (SkkModeFunc, 1);
 	func->listener =listener;
 	func->user_data = user_data;
@@ -500,6 +565,8 @@ skk_mode_add_j_status_listener (SkkMode *mode, SkkModeListener listener, gpointe
 	SkkModeFunc *func;
 	if (!mode)
 		return;
+	if (!listener)
+		return;
 	func = g_new (SkkModeFunc, 1);
 	func->listener = listener;
 	func->user_data = user_data;
@@ -513,6 +580,8 @@ skk_mode_add_query_status_listener (SkkMode *mode, SkkModeListener listener, gpo
 	SkkModeFunc *func;
 	if (!mode)
 		return;
+	if (!listener)
+		return;
 	func = g_new (SkkModeFunc, 1);
 	func->listener = listener;
 	func->user_data = user_data;
@@ -520,6 +589,72 @@ skk_mode_add_query_status_listener (SkkMode *mode, SkkModeListener listener, gpo
 	return;
 }
 
+void
+skk_mode_set_latin_str (SkkMode *mode, const gchar *str)
+{
+	if (!mode || !str)
+		return;
+	if (mode->latin_str)
+		g_free (mode->latin_str);
+	mode->latin_str = g_strdup (str);
+	return;
+}
+
+void
+skk_mode_set_hirakana_str (SkkMode *mode, const gchar *str)
+{
+	if (!mode || !str)
+		return;
+	if (mode->hirakana_str)
+		g_free (mode->hirakana_str);
+	mode->hirakana_str = g_strdup (str);
+	return;
+}
+
+void
+skk_mode_set_katakana_str (SkkMode *mode, const gchar *str)
+{
+	if (!mode || !str)
+		return;
+	if (mode->katakana_str)
+		g_free (mode->katakana_str);
+	mode->katakana_str = g_strdup (str);
+	return;
+}
+
+void
+skk_mode_set_jisx0208_latin_str (SkkMode *mode, const gchar *str)
+{
+	if (!mode || !str)
+		return;
+	if (mode->jisx0208_latin_str)
+		g_free (mode->jisx0208_latin_str);
+	mode->jisx0208_latin_str = g_strdup (str);
+	return;
+}
+
+void
+skk_mode_set_jisx0201_kana_str (SkkMode *mode, const gchar *str)
+{
+	if (!mode || !str)
+		return;
+	if (mode->jisx0201_kana_str)
+		g_free (mode->jisx0201_kana_str);
+	mode->jisx0201_kana_str = g_strdup (str);
+	return;
+}
+
+void
+skk_mode_set_abbrev_str (SkkMode *mode, const gchar *str)
+{
+	if (!mode || !str)
+		return;
+	if (mode->abbrev_str)
+		g_free (mode->abbrev_str);
+	mode->abbrev_str = g_strdup (str);
+	return;
+}
+
 gboolean
 skk_mode_remove_status_listener (SkkMode *mode, SkkModeListener listener)
 {
