@@ -32,7 +32,7 @@
  *
  */
 
-/* $Id: skkmode.h,v 1.1.1.1 2002/04/06 02:23:30 famao Exp $ */
+/* $Id: skkmode.h,v 1.3 2002/11/09 16:56:49 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -40,7 +40,7 @@
 #ifndef HAVE_SKKMODE_H
 #define HAVE_SKKMODE_H
 #include <glib.h>
-#include "skktypes.h"
+#include <skktypes.h>
 
 typedef struct _SkkMode SkkMode;
 
@@ -51,6 +51,12 @@ struct _SkkMode
 	gchar *query_prefix;
 	gchar *query_suffix;
 	gchar *status_buf;
+	gchar *latin_str;
+	gchar *hirakana_str;
+	gchar *katakana_str;
+	gchar *jisx0208_latin_str;
+	gchar *jisx0201_kana_str;
+	gchar *abbrev_str;
 	gboolean katakana;
 	gboolean jisx0201_kana;
 	SkkStatus status;
@@ -66,6 +72,7 @@ struct _SkkMode
 	void (*set_status)(SkkMode *mode,SkkStatus status);
 	void (*set_j_status) (SkkMode *mode, SkkJStatus jstatus);
 	void (*set_query_status) (SkkMode *mode, SkkQueryStatus status);
+	void (*prepare_mark) (SkkMode *mode);
 	gchar* (*get_status_string) (SkkMode *mode);
 	SkkStatus (*get_status) (SkkMode *mode);
 	SkkJStatus (*get_j_status) (SkkMode *mode);
@@ -106,4 +113,10 @@ gboolean skk_mode_remove_status_listener (SkkMode *mode, SkkModeListener listene
 gboolean skk_mode_remove_j_status_listener (SkkMode *mode, SkkModeListener listener);
 gboolean skk_mode_remove_query_status_listener (SkkMode *mode, SkkModeListener listener);
 
+void skk_mode_set_latin_str (SkkMode *mode, const gchar *str);
+void skk_mode_set_hirakana_str (SkkMode *mode, const gchar *str);
+void skk_mode_set_katakana_str (SkkMode *mode, const gchar *str);
+void skk_mode_set_jisx0208_latin_str (SkkMode *mode, const gchar *str);
+void skk_mode_set_jisx0201_kana_str (SkkMode *mode, const gchar *str);
+void skk_mode_set_abbrev_str (SkkMode *mode, const gchar *str);
 #endif /* HAVE_SKKCONV_MODE_H */
