@@ -32,7 +32,7 @@
  *
  */
 
-/* $Id: skkquery.c,v 1.2.2.1 2002/04/15 21:29:56 famao Exp $ */
+/* $Id: skkquery.c,v 1.8 2002/11/09 16:56:49 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -40,16 +40,16 @@
 #include <string.h>
 #include "config.h"
 #include "skkquery.h"
+#include "skkqueryimpl.h"
 #include "skkutils.h"
-#include "skkdictimpl.h"
-#include "skkldictimpl.h"
-#include "skklookimpl.h"
 
+#if 0
 static gboolean initialized = FALSE;
 static GList *plugin_list = NULL;
+#endif
 
 static SkkQueryPlugin *
-plugin_new (const gchar *filename)
+plugin_new (SkkQuery *query, const gchar *filename)
 {
 	SkkQueryPlugin *plugin;
 	SkkQueryFunctionImpl *impl;
@@ -69,67 +69,116 @@ plugin_new (const gchar *filename)
 		g_free (plugin);
 		return NULL;
 	}
-	impl = plugin->impl_init ();
+	impl = plugin->impl_init (query);
 	plugin->type = impl->get_type ();
 	plugin->info = impl->get_info ();
 	impl->destroy (impl);
+	plugin->num_use = 0;
 	return plugin;
 }
 
 static void
-query_init (void)
+query_init (SkkQuery *query, const gchar *path)
 {
 	SkkQueryPlugin *plugin;
 	DIR *d;
 	struct dirent *dent;
-	if ((d = opendir (PACKAGE_PLUGIN_DIR)) == NULL) {
+	if (!query)
+		return;
+	if ((d =  opendir (path ? path : PACKAGE_PLUGIN_DIR)) == NULL) {
 		return;
 	}
-	/* TODO dlopen */
+	/* FIXME use g_module_build_path ? */
 	while ((dent = readdir (d)) != NULL) {
 		if (!strncmp (dent->d_name + strlen (dent->d_name) - 3, ".so", 3)) {
-			gchar *file = g_strconcat (PACKAGE_PLUGIN_DIR, "/" , dent->d_name, NULL);
-			plugin = plugin_new (file);
-			if (plugin)
-				plugin_list = g_list_append (plugin_list, plugin);
+			gchar *file;
+			if (path) {
+				file = g_strconcat (path, "/", dent->d_name, NULL);
+			} else {
+				file = g_strconcat (PACKAGE_PLUGIN_DIR, "/", dent->d_name, NULL);
+			}
+			plugin = plugin_new (query, file);
+			if (plugin) {
+				query->plugin_list = g_list_append (query->plugin_list, plugin);
+			}
 			g_free (file);
 		}
 	}
 	closedir (d);
-	initialized = TRUE;
+	query->initialized = TRUE;
 	return;
 }
 
+static void
+query_finalize_func  (gpointer free_item, gpointer user_data)
+{
+	SkkQueryPlugin *plugin = (SkkQueryPlugin *)free_item;
+	if (!plugin)
+		return;
+	if (plugin->file)
+		g_free (plugin->file);
+	if (plugin->handle)
+		g_module_close (plugin->handle);
+	g_free (plugin);
+}
+
+static void
+query_finalize (SkkQuery *query)
+{
+	if (!query || !query->plugin_list)
+		return;
+	skk_utils_list_free (query->plugin_list, TRUE, query_finalize_func, NULL);
+	query->plugin_list = NULL;
+	query->initialized = FALSE;
+}
+
 SkkQuery *
 skk_query_new (void)
 {
 	SkkQuery *ret;
-	if (!initialized) {
-		query_init ();
-	}
 	ret = g_new (SkkQuery, 1);
 	memset (ret, 0, sizeof (SkkQuery));
+	ret->plugin_list = NULL;
+	ret->initialized = FALSE;
 	ret->query_list = NULL;
 	ret->ref_count++;
 	ret->cur_item = NULL;
+	ret->conf = NULL;
+	query_init (ret, NULL);
+	return ret;
+}
+
+SkkQuery *
+skk_query_new_with_path (const gchar *path)
+{
+	SkkQuery *ret;
+	ret = g_new0 (SkkQuery, 1);
+	ret->plugin_list = NULL;
+	ret->initialized = FALSE;
+	ret->query_list = NULL;
+	ret->ref_count++;
+	ret->cur_item = NULL;
+	ret->conf = NULL;
+	query_init (ret, path);
 	return ret;
 }
 
 SkkQueryItem *
-skk_query_item_new (SkkQueryType type)
+skk_query_item_new (SkkQuery *query, SkkQueryType type)
 {
-	SkkQueryItem *item;
+	SkkQueryItem *item = NULL;
 	GList *l;
-	item = g_new (SkkQueryItem, 1);
-	memset (item, 0, sizeof (SkkQueryItem));
-	item->type = type;
-
-	if (!initialized)
-		query_init ();
-	for (l = plugin_list; l ; l = g_list_next (l)) {
+	if (!query)
+		return NULL;
+	if (!query->initialized)
+		query_init (query, NULL);
+	for (l = query->plugin_list; l ; l = g_list_next (l)) {
 		SkkQueryPlugin *p = l->data;
 		if (p->type == type) {
-			item->impl = p->impl_init ();
+			item = g_new0 (SkkQueryItem, 1);
+			item->type = type;
+			item->impl = p->impl_init (query);
+			p->num_use++;
 			break;
 		}
 	}
@@ -162,6 +211,10 @@ skk_query_destroy (SkkQuery *query)
 		skk_utils_list_free (query->query_list, TRUE, query_item_destroy_func , NULL);
 		query->query_list = NULL;
 	}
+	if (query->conf) {
+		skk_conf_unref (query->conf);
+	}
+	query_finalize (query);
 	g_free (query);
 	return;
 }
@@ -196,6 +249,26 @@ skk_query_set_with_type (SkkQuery *query, SkkQueryType type)
 	return FALSE;
 }
 
+void
+skk_query_remove_item (SkkQuery *query, SkkQueryType type)
+{
+	GList *l;
+	SkkQueryItem *item;
+	/* FIXME */
+	if (!query)
+		return;
+	for (l = query->query_list; l ; l = g_list_next (l)) {
+		item = l->data;
+		if (item && item->type == type) {
+			query_item_destroy_func (item, NULL);
+			query->query_list = g_list_remove_link (query->query_list, l);
+			g_list_free_1 (l);
+			break;
+		}
+	}
+	return;
+}
+
 gboolean
 skk_query_set_next (SkkQuery *query)
 {
@@ -299,5 +372,50 @@ skk_query_unref (SkkQuery *query)
 	if (!query)
 		return;
 	query->ref_count--;
+	if (!query->ref_count) {
+		skk_query_destroy (query);
+	}
 	return;
 }
+
+void
+skk_query_set_conf (SkkQuery *query, SkkConf *conf)
+{
+	if (!query)
+		return;
+	if (query->conf) {
+		skk_conf_unref (conf);
+	}
+	query->conf = conf;
+	skk_conf_ref (query->conf);
+}
+
+gchar *
+skk_query_get_conf_string_by_name (SkkQuery *query, const gchar *name)
+{
+	if (!query)
+		return NULL;
+	if (!query->conf)
+		return NULL;
+	return skk_conf_get_string_by_name (query->conf, name);
+}
+
+gint
+skk_query_get_conf_num_by_name (SkkQuery *query, const gchar *name)
+{
+	if (!query)
+		return -1;
+	if (!query->conf)
+		return -1;
+	return skk_conf_get_num_by_name (query->conf, name);
+}
+
+gboolean
+skk_query_get_conf_bool_by_name (SkkQuery *query, const gchar *name)
+{
+	if (!query)
+		return FALSE;
+	if (!query->conf)
+		return FALSE;
+	return skk_conf_get_bool_by_name (query->conf, name);
+}
