@@ -32,7 +32,7 @@
  *
  */
 
-/* $Id: skkquery.h,v 1.1.1.1.2.1 2002/04/15 21:29:56 famao Exp $ */
+/* $Id: skkquery.h,v 1.5 2002/11/09 16:56:49 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -41,20 +41,22 @@
 
 #include <glib.h>
 #include <gmodule.h>
-#include "skktypes.h"
-#include "skkqueryimpl.h"
+#include <skktypes.h>
+#include <skkconf.h>
 
 typedef struct _SkkQuery SkkQuery;
 typedef struct _SkkQueryPlugin SkkQueryPlugin;
 typedef struct _SkkQueryItem SkkQueryItem;
+typedef struct _SkkQueryFunctionImpl SkkQueryFunctionImpl;
 
 struct _SkkQueryPlugin
 {
 	SkkQueryType type;
 	gchar *file;
-	const gchar *info;
+	gchar *info;
+	gint num_use;
 	GModule *handle;
-	SkkQueryFunctionImpl* (*impl_init) (void);
+	SkkQueryFunctionImpl* (*impl_init) (SkkQuery *query);
 };
 
 struct _SkkQueryItem
@@ -67,16 +69,21 @@ struct _SkkQuery
 {
 	gint ref_count;
 	gint cur_list_num;
+	gboolean initialized;
 	GList *query_list;
+	GList *plugin_list;
 	SkkQueryItem *cur_item;
+	SkkConf *conf;
 };
 
 #define skk_query_set_first(x) skk_query_set_nth((x), 0)
 
 SkkQuery* skk_query_new (void);
-SkkQueryItem* skk_query_item_new (SkkQueryType type);
+SkkQuery* skk_query_new_with_path (const gchar *path);
+SkkQueryItem* skk_query_item_new (SkkQuery *query, SkkQueryType type);
 void skk_query_destroy (SkkQuery *query);
 void skk_query_add_item (SkkQuery *query, SkkQueryItem *item);
+void skk_query_remove_item (SkkQuery *query, SkkQueryType type);
 
 gboolean skk_query_set_with_type (SkkQuery *query, SkkQueryType type);
 gboolean skk_query_set_next (SkkQuery *query);
@@ -90,4 +97,9 @@ void skk_query_add (SkkQuery *query, const gchar *key, const gchar *okuri, const
 void skk_query_ref (SkkQuery *query);
 void skk_query_unref (SkkQuery *query);
 
+void skk_query_set_conf (SkkQuery *query, SkkConf *conf);
+gchar *skk_query_get_conf_string_by_name (SkkQuery *query, const gchar *name);
+gboolean skk_query_get_conf_bool_by_name (SkkQuery *query, const gchar *name);
+gint skk_query_get_conf_num_by_name (SkkQuery *query, const gchar *name);
+
 #endif
