@@ -32,7 +32,7 @@
  *
  */
 
-/* $Id: skktypes.h,v 1.4.2.3 2002/04/12 21:54:11 famao Exp $ */
+/* $Id: skktypes.h,v 1.9 2002/11/09 16:56:49 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -91,7 +91,9 @@ typedef enum {
 	SKKQUERY_FILE = 0,
 	SKKQUERY_LOCAL,
 	SKKQUERY_LOOK,
-	SKKQUERY_SERVER
+	SKKQUERY_SERVER,
+	SKKQUERY_CANNA_SERVER,
+	SKKQUERY_POBOX_SERVER,
 }SkkQueryType;
 
 typedef enum {
@@ -123,13 +125,53 @@ typedef enum {
 } SkkKeyResult;
 
 typedef enum {
-	SKKCONF_FUGA
-}SkkConfBool;
+	SKKCONF_TYPE_BOOL,
+	SKKCONF_TYPE_STRING,
+	SKKCONF_TYPE_NUM,
+	SKKCONF_TYPE_INVALID
+}SkkConfType;
 
 typedef enum {
-	SKKCONF_HOGE
-}SkkConfString;
+	SKKCONF_ITEM_INVALID = 0,
+	/* Bool */
+	SKKCONF_BOOL_DABBREV_LIKE_COMPLETION,
+	SKKCONF_BOOL_DATE_AD,
+	SKKCONF_BOOL_DELETE_IMPLIES_KAKUTEI,
+	SKKCONF_BOOL_DELETE_OKURI_WHEN_QUIT,
+	SKKCONF_BOOL_ECHO,
+	SKKCONF_BOOL_EGG_LIKE_NEWLINE,
+	SKKCONF_BOOL_USE_CANNA,
+	SKKCONF_BOOL_USE_DICT,
+	SKKCONF_BOOL_USE_LDICT,
+	SKKCONF_BOOL_USE_LOOK,
+	SKKCONF_BOOL_USE_POBOX,
+	/* Num */
+	SKKCONF_NUM_CANDIDATE_COUNT,
+	/* String */
+	SKKCONF_STRING_JISYO,
+	SKKCONF_STRING_MODE_ABBREV,
+	SKKCONF_STRING_MODE_HIRAKANA,
+	SKKCONF_STRING_MODE_JISX0201_KANA,
+	SKKCONF_STRING_MODE_JISX0208_LATIN,
+	SKKCONF_STRING_MODE_KATAKANA,
+	SKKCONF_STRING_MODE_LATIN,
+	SKKCONF_STRING_OKURI_JISYO,
+	SKKCONF_STRING_POBOX_SERVER_HOST,
+	SKKCONF_STRING_POBOX_SERVER_PORT,
+	SKKCONF_STRING_SERVER_HOST,
+	SKKCONF_STRING_SERVER_PORT
+}SkkConfItemType;
 
+/* skkconv_kana.c */
+typedef enum {
+	SKKCONV_TYPE_NONE = 0,
+	SKKCONV_TYPE_NORMAL,
+	SKKCONV_TYPE_FUNC,
+	SKKCONV_TYPE_USER,
+	SKKCONV_TYPE_USER_FUNC
+}SkkConvRuleType;
+
+/* skkfunc.c */
 typedef enum {
 	SKKFUNC_TRANSLATE_BEGIN = 1,
 	SKKFUNC_BACKWARD_DELETE,
@@ -140,6 +182,8 @@ typedef enum {
 	SKKFUNC_COMPLETE_NEXT,
 	SKKFUNC_COMPLETE_PREV,
 	SKKFUNC_COMPLETE_TRANSLATE,
+	SKKFUNC_CURSOR_LEFT,
+	SKKFUNC_CURSOR_RIGHT,
 	SKKFUNC_HALFWIDTH_KANA_SWITCH,
 	SKKFUNC_KAKUTEI,
 	SKKFUNC_KANA_SWITCH,
@@ -149,4 +193,21 @@ typedef enum {
 	SKKFUNC_TODAY,
 	SKKFUNC_UNDO
 }SkkFunction;
+
+/* skkel.c */
+
+typedef enum {
+	SKKLISP_NIL = 0,
+	SKKLISP_TRUE,
+	SKKLISP_CONS,
+	SKKLISP_CAR,
+	SKKLISP_CDR,
+	SKKLISP_LIST,
+	SKKLISP_EQ,
+	SKKLISP_ATOM,
+	SKKLISP_SET,
+	SKKLISP_LET,
+	SKKLISP_STRING
+}SkkLispSymbolType;
+
 #endif /* HAVE_SKKTYPES_H */
