@@ -32,7 +32,7 @@
  *
  */
 
-/* $Id: skkutils.h,v 1.3 2002/04/11 12:44:44 famao Exp $ */
+/* $Id: skkutils.h,v 1.7 2002/11/09 16:56:49 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -42,19 +42,26 @@
 #include <glib.h>
 
 /* String */
+
 gint skk_utils_strlen (const gchar *str);
 gint skk_utils_charbytes (const gchar *str);
 gint skk_utils_charbytes_nth (const gchar *str, guint nth);
+gint skk_utils_charbytes_nth_all (const gchar *str, guint nth);
 gint skk_utils_last_charbytes (const gchar *str);
+gint skk_utils_last_charbytes_nth_all (const gchar *str, guint nth);
 gboolean skk_utils_is_kana (const gchar *str);
 gboolean skk_utils_is_hiragana (const gchar *str);
 gboolean skk_utils_is_katakana (const gchar *str);
 gboolean skk_utils_is_halfwidth_katakana (const gchar *str);
 gchar* skk_utils_get_char (const gchar *str, gboolean with_dakuon);
+gchar* skk_utils_remove_char_nth (const gchar *str, guint nth);
+gchar* skk_utils_escape_char (const gchar *str);
 
 /* List */
 void skk_utils_list_free (GList *list, gboolean need_free, GFunc func, gpointer user_data);
-GList* skk_utils_list_merge_string (GList *list, GList *list2);
+GList* skk_utils_list_merge_string (GList *list, GList *list2, gboolean need_free);
 gchar* skk_utils_list_to_string_all (GList *list, gchar *delim);
 
+/* Convert */
+gchar* skk_utils_utf8_to_eucjp (const gchar *str);
 #endif /* HAVE_SKKUTILS_H */
