@@ -33,7 +33,7 @@
  *
  */
 
-/* $Id: skk.c,v 1.9.2.3 2002/04/24 22:13:01 famao Exp $ */
+/* $Id: skk.c,v 1.14 2002/11/09 16:56:50 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -86,13 +86,17 @@ Sun Microsystems, Inc. or its licensors is granted.
 #include <sys/types.h>
 #include <dlfcn.h>
 #include <pthread.h>
+#include <pwd.h>
+#include <assert.h>
+#include <errno.h>
 #include "csconv.h"
 #include "skkcomm.h"
 #include "skkbuffer.h"
+#include "skkconv_kana.h"
 #include "skkconv_jisx0208_latin.h"
 #include "skkkeymap.h"
 #include "skkkeysym.h"
-
+#include "skkutils.h"
 /*
  * header files for SunIM library
  */
@@ -166,6 +170,9 @@ static void skk_lookup_listener (SkkBuffer *buf, gchar **label, gchar **candidat
 static void skk_codetable_listener (SkkBuffer *buf, gpointer user_data);
 static void skk_status_listener (SkkMode *mode, gint status, gpointer user_data);
 
+static void skk_add_rule_listener (SkkConf *conf, const gchar *key, const gchar *hira,
+		const gchar *kata, const gchar *append, gpointer user_data);
+
 /* Preedit */
 static void skk_preedit_done (iml_session_t * s);
 static void skk_preedit_draw (iml_session_t * s, unsigned char *str);
@@ -215,6 +222,7 @@ static IMText *skk_strtoimtext (iml_session_t * s, int nseg, int *nb,
 			 unsigned char **strs, int *feedback_type);
 static UTFCHAR *skk_strtoutf (const unsigned char *str);
 
+static Bool skk_drop_privilege (const char *username);
 
 typedef enum
 {
@@ -249,6 +257,7 @@ struct _SkkDesktopData
 	SkkFunc *skkfunc;
 	SkkQuery *skkquery;
 	SkkConf *skkconf;
+	SkkConvRule *skkconv;
 };
 
 
@@ -504,6 +513,7 @@ static Bool
 if_skk_OpenDesktop (iml_desktop_t * desktop, IMArgList args, int num_args)
 {
 	int i;
+	Bool drop_priv;
 	SkkDesktopData *d;
 
 #ifdef DEBUG
@@ -568,14 +578,41 @@ if_skk_OpenDesktop (iml_desktop_t * desktop, IMArgList args, int num_args)
 				break;
 		}
 	}
+	drop_priv = skk_drop_privilege (desktop->user_name);
 	d = g_new0 (SkkDesktopData, 1);
 	d->skkfunc = skk_func_new ();
 	d->skkquery = skk_query_new ();
 	d->skkconf = skk_conf_new ();
+	d->skkconv = skk_conv_rule_new ();
+	skk_conf_add_rule_listener (d->skkconf, skk_add_rule_listener, d->skkconv);
+	if (drop_priv) {
+#ifdef DEBUG
+		printf ("Drop Privilege\n");
+#endif
+		skk_conf_load_rc (d->skkconf, desktop->user_name);
+	} else {
+#ifdef DEBUG
+		printf ("Drop Privilege Failed\n");
+#endif
+	}
+	skk_query_set_conf (d->skkquery, d->skkconf);
+
 	/* TODO configurable */
-	skk_query_add_item (d->skkquery, skk_query_item_new (SKKQUERY_LOCAL));
-	skk_query_add_item (d->skkquery, skk_query_item_new (SKKQUERY_SERVER));
-	skk_query_add_item (d->skkquery, skk_query_item_new (SKKQUERY_LOOK));
+	if (skk_conf_get_bool_by_name (d->skkconf, "skk-plugin-use-ldict")) {
+		skk_query_add_item (d->skkquery, skk_query_item_new (d->skkquery, SKKQUERY_LOCAL));
+	}
+	if (skk_conf_get_bool_by_name (d->skkconf, "skk-plugin-use-dict")) {
+		skk_query_add_item (d->skkquery, skk_query_item_new (d->skkquery, SKKQUERY_SERVER));
+	}
+	if (skk_conf_get_bool_by_name (d->skkconf, "skk-plugin-use-canna")) {
+		skk_query_add_item (d->skkquery, skk_query_item_new (d->skkquery, SKKQUERY_CANNA_SERVER));
+	}
+	if (skk_conf_get_bool_by_name (d->skkconf, "skk-plugin-use-look")) {
+		skk_query_add_item (d->skkquery, skk_query_item_new (d->skkquery, SKKQUERY_LOOK));
+	}
+	if (skk_conf_get_bool_by_name (d->skkconf, "skk-plugin-use-pobox")) {
+		skk_query_add_item (d->skkquery, skk_query_item_new (d->skkquery, SKKQUERY_POBOX_SERVER));
+	}
 	desktop->specific_data = (void *)d;
 	return True;
 }
@@ -615,10 +652,22 @@ if_skk_CloseDesktop (iml_desktop_t * desktop)
 	if (d->skkconf) {
 		skk_conf_destroy (d->skkconf);
 	}
+	if (d->skkconv) {
+		skk_conv_rule_destroy (d->skkconv);
+	}
 	g_free (d);
 	return True;
 }
 
+static void
+skk_add_rule_listener (SkkConf *conf , const gchar *key,
+		const gchar *hira, const gchar *kata, const gchar *append, gpointer user_data)
+{
+	SkkConvRule *rule = (SkkConvRule *)user_data;
+	skk_conv_add_rule (rule, skk_conv_rule_item_new_with_value (key, hira, kata, append));
+	return;
+}
+
 static void
 skk_result_listener (SkkBuffer *buf, gchar *value, gpointer user_data)
 {
@@ -740,6 +789,7 @@ if_skk_CreateSC (iml_session_t * s, IMArgList args, int num_args)
 	s->specific_data = (void *) ses;
 	skk_buffer_set_query (buf, d->skkquery);
 	skk_buffer_set_conf (buf, d->skkconf);
+	skk_buffer_set_rule (buf, d->skkconv);
 	skk_buffer_add_result_listener (buf, skk_result_listener, (gpointer) s);
 	skk_buffer_add_preedit_listener (buf, skk_preedit_listener, (gpointer) s);
 	skk_mode_add_status_listener (skkbuf_mode (buf), skk_status_listener, (gpointer) s);
@@ -1182,7 +1232,7 @@ trans:
 			candidate = skk_buffer_get_nth_candidate (buf, buf->candidate_count + num);
 			if (candidate) {
 				skk_commit_string (s, candidate);
-				skk_buffer_add_dict_with_value (buf, candidate);
+				skk_buffer_add_dict_with_value (buf, buf->has_num ? skkbuf_numb (buf) : candidate);
 				skk_buffer_clear (buf);
 				g_free (candidate);
 			}
@@ -1647,13 +1697,57 @@ skk_preedit_draw (iml_session_t *s, unsigned char *str)
 	iml_inst *lp;
 	IMText *p;
 	SkkSessionData *sses = skk_session_data (s);
+	SkkBuffer *buf = skk_session_buf (s);
+	SkkCursor *cursor = skkbuf_cursor (buf);
 
 	if (sses->preedit_start == False) {
 		lp = s->If->m->iml_make_preedit_start_inst (s);
 		s->If->m->iml_execute (s, &lp);
 		sses->preedit_start = True;
 	}
-	{
+	if (cursor->revLen) {
+		/* Create IMText with feedbacks */
+		int nb[3], fts[3];
+		unsigned char *strs[3];
+		nb[0] = cursor->revPos;
+		nb[1] = cursor->revLen;
+		nb[2] = strlen (str) - cursor->revPos - cursor->revLen;
+		fts[0] = SKK_Feedback_Input;
+		fts[1] = SKK_Feedback_Strong;
+		fts[2] = SKK_Feedback_Input;
+		strs[0] = str;
+		strs[1] = str + cursor->revPos;
+		strs[2] = str + cursor->revPos + cursor->revLen;
+		g_message ("%s %s %s %d %d %d", strs[0], strs[1], strs[2],
+				nb[0], nb[1], nb[2]);
+		if (nb[2] == 0) {
+			p = skk_strtoimtext (s, 2, nb, strs, fts);
+		} else {
+			p = skk_strtoimtext (s, 3, nb, strs, fts);
+		}
+	} else if (cursor->caret){
+		int nb[3], fts[3];
+		unsigned char *strs[3];
+		int cur = skk_utils_last_charbytes_nth_all (str, cursor->caret);
+		int charlen = skk_utils_strlen (str);
+		int cur_pos = skk_utils_charbytes_nth (str, charlen - cursor->caret - 1);
+		nb[0] = strlen (str) - cur;
+		nb[1] = cur_pos;
+		nb[2] = strlen (str) - nb[0] - nb[1];
+		fts[0] = SKK_Feedback_Input;
+		fts[1] = SKK_Feedback_Strong;
+		fts[2] = SKK_Feedback_Input;
+		strs[0] = str;
+		strs[1] = str + strlen (str) - cur;
+		strs[2] = str + nb[0] +  nb[1];
+		g_message ("caret %s %s %s %d %d %d", strs[0], strs[1], strs[2],
+				nb[0], nb[1], nb[2]);
+		if (nb[2] == 0) {
+			p = skk_strtoimtext (s, 2, nb, strs, fts);
+		} else {
+			p = skk_strtoimtext (s, 3, nb, strs, fts);
+		}
+	} else {
 		/* Create IMText with feedbacks */
 		int nb[1],fts[1];
 		unsigned char *strs[1];
@@ -1664,8 +1758,14 @@ skk_preedit_draw (iml_session_t *s, unsigned char *str)
 	}
 	lp = s->If->m->iml_make_preedit_erase_inst (s);
 	s->If->m->iml_execute (s,&lp);
-	lp = s->If->m->iml_make_preedit_draw_inst (s,p);
+	lp = s->If->m->iml_make_preedit_draw_inst (s, p);
 	s->If->m->iml_execute (s, &lp);
+#if 0
+	if (cursor->caret) {
+		lp = s->If->m->iml_make_preedit_caret_inst (s, cursor->caret);
+		s->If->m->iml_execute (s, &lp);
+	}
+#endif
 }
 
 /* PreEditing Done */
@@ -2032,14 +2132,12 @@ skk_process_auxevent (iml_session_t *s, IMAuxDrawCallbackStruct *aux)
 	for (i = 0; i < aux->count_integer_values; i++) {
 		switch (aux->integer_values[i]) {
 			case SKK_MODE_JMODE_HIRA:
-				skk_buffer_clear (buf);
 				skk_buffer_set_katakana (buf, FALSE);
-				skk_status_draw (s);
+				skk_buffer_clear (buf);
 				break;
 			case SKK_MODE_JMODE_KATA:
-				skk_buffer_clear (buf);
 				skk_buffer_set_katakana (buf, TRUE);
-				skk_status_draw (s);
+				skk_buffer_clear (buf);
 				break;
 			case SKK_MODE_LATIN:
 				skk_buffer_clear (buf);
@@ -2048,7 +2146,6 @@ skk_process_auxevent (iml_session_t *s, IMAuxDrawCallbackStruct *aux)
 			case SKK_MODE_JISX0201_KANA:
 				skk_buffer_clear (buf);
 				skk_buffer_set_jisx0201_katakana (buf, TRUE);
-				skk_status_draw (s);
 				break;
 			case SKK_MODE_JISX0208_LATIN:
 				skk_buffer_clear (buf);
@@ -2165,3 +2262,54 @@ skk_status_done (iml_session_t *s)
 	}
 	return;
 }
+
+static Bool
+skk_drop_privilege (const char *username)
+{
+	struct passwd *pw;
+	uid_t uid;
+	gid_t gid;
+	if (!username)
+		goto error;
+	if ((pw = getpwnam (username)) == NULL) {
+		goto error;
+	} else {
+		uid = pw->pw_uid;
+		gid = pw->pw_gid;
+		if (uid < 500)
+			goto error;
+	}
+	if (setregid (gid, gid) < 0) {
+#ifdef DEBUG
+		printf ("setregid fail %s\n", strerror (errno));
+#endif
+		return False;
+	}
+	if (setreuid (uid, uid) < 0) {
+#ifdef DEBUG
+		printf ("setreuid fail %s\n", strerror (errno));
+#endif
+		return False;
+	}
+	return True;
+
+error:
+	if ((pw = getpwnam ("nobody")) != NULL) {
+		gid = pw->pw_gid;
+		uid = pw->pw_uid;
+		if (setregid (gid, gid) < 0) {
+#ifdef DEBUG
+			printf ("setregid fail %s\n", strerror (errno));
+#endif
+		}
+		if (setreuid (uid, uid) < 0) {
+#ifdef DEBUG
+			printf ("setreuid fail %s\n", strerror (errno));
+#endif
+		}
+	} else {
+		/* not reached */
+		assert (0);	
+	}
+	return False;
+}
